(function() {
	'use strict';

	if (typeof Function.prototype.bind != 'function') {
		Function.prototype.bind = function bind(obj) {
			var args = Array.prototype.slice.call(arguments, 1), self = this, nop = function() {
			}, bound = function() {
				return self.apply(this instanceof nop ? this : (obj || {}), args.concat(Array.prototype.slice
						.call(arguments)));
			};
			nop.prototype = this.prototype || {};
			bound.prototype = new nop();
			return bound;
		};
	}
	var billingApp = angular.module("billingApp", []);

	billingApp.controller("billingController", [ "$scope", "billingService", function($scope, billingService) {
		$scope.initialize = function() {
			var detailedReport = 0;
			if (cvUtil.getTopWindow().document.URL.indexOf("billingDetails.jsp") > 0) {
				detailedReport = 1;
			}
			var params = {
				customerId : $scope.getParameter("customerId", -1),
				month : $scope.getParameter("month", 1),
				year : $scope.getParameter("year", 0),
				quarter : $scope.getParameter("quarter", 0),
				send : $scope.getParameter("send", "false"),
				type : $scope.getParameter("type", 0),
				detailedReport : detailedReport
			};

			billingService.getRoyaltyReport(params).then(function(data) {
				if (data.data === "noView") {
					window.open(cvUtil.getContextPath() + "/applications", "_self", "", "true");
				} else {

					$scope.aggregatorResponse = [];
					if ($scope.isPartner()) {
						$scope.partnerCapacityTable = data.data.data.partnerCapacityTable;
						$scope.partnerClientTable = data.data.data.partnerClientTable;
						$scope.partnerName = data.data.data.partnerName;
						$scope.partnerBillCycle = data.data.data.partnerBillCycle;
						$scope.partnerAmountDue = data.data.data.partnerAmountDue;
						$scope.partnerTotalCost = data.data.data.partnerTotalCost;
						$scope.partnerDiscount = data.data.data.partnerDiscount;
						$scope.partnerDiscountPercentage = data.data.data.partnerDiscountPercentage;
						$scope.aggregatorResponse = data.data.listData;
						$scope.partnerVolumeDiscountApplicable = data.data.data.partnerVolumeDiscountApplicable;
						$scope.unadjustedAmount = data.data.data.unadjustedAmount;
						$scope.discountPercentage = data.data.data.discountPercentage;
						$scope.discountedAmount = data.data.data.discountedAmount;
						$scope.adjustedAmount = data.data.data.adjustedAmount;
					} else {
						$scope.aggregatorResponse[0] = data.data;
					}

					if ($scope.isQuarterly()) {
						$scope.quarterMonth = $scope.aggregatorResponse[0].listData[0].header.month;
						$scope.quarterMonth2 = $scope.aggregatorResponse[0].listData[0].header2.month;
						$scope.quarterMonth3 = $scope.aggregatorResponse[0].listData[0].header3.month;
					}

					$scope.billingCycle = $scope.getBillingCycle();
					$scope.customerName = $scope.getCustomerName(data);
					$scope.useAsPO = data.data.data.useAsPO;
					$scope.currencyCode = data.data.data.currencyCode;
					$scope.legalText = $scope.getLegalText();
					$scope.signatureText = $scope.getSignatureText();
				}
			}, function() {

			});
		};

		$scope.getLegalText = function() {
			var legalText = "";
			if ($scope.isPartner() === true) {
				legalText = cvUtil.cvLocalize("Legal_Text", cvUtil.cvLocalize("Partner"));
			} else {
				legalText = cvUtil.cvLocalize("Legal_Text", cvUtil.cvLocalize("Service_Provider"));
			}
			return legalText;
		};

		$scope.getSignatureText = function() {
			var signatureText = "";
			if ($scope.isPartner() === true) {
				signatureText = cvUtil.cvLocalize("Signature_Blob_Line1", cvUtil.cvLocalize("Partner"));
			} else {
				signatureText = cvUtil.cvLocalize("Signature_Blob_Line1", cvUtil.cvLocalize("Service_Provider"));
			}
			return signatureText;
		};

		$scope.getCustomerName = function(data) {
			var customerName = "";
			if ($scope.isAggregator() === false && $scope.isPartner() === false) {
				customerName = data.data.listData[0].header.custNameValue;
			} else {
				if ($scope.isAggregator() === true) {
					customerName = data.data.data.aggregatorName;
				} else {
					customerName = data.data.data.partnerName;
				}
			}

			return customerName;
		};

		$scope.getBillingCycle = function() {
			var billingCycle = "";
			if ($scope.isQuarterly() === true) {
				billingCycle = cvUtil.cvLocalize("Quarterly");
			} else {
				billingCycle = cvUtil.cvLocalize("Monthly");
			}
			return billingCycle;
		};

		$scope.isBillingGroup = function() {
			var type = $scope.getParameter("type", 0);
			if (parseInt(type) === 0) {
				return true;
			}

			return false;

		};
		$scope.isPartner = function() {
			var type = $scope.getParameter("type", 0);
			if (parseInt(type) === 2) {
				return true;
			}

			return false;

		};
		$scope.isAggregator = function() {
			var type = $scope.getParameter("type", 0);
			if (parseInt(type) === 1) {
				return true;
			}

			return false;

		};
		$scope.hideSummarySection = function() {
			if ($scope.isPartner()) {
				var hideSummary = $scope.getParameter("hideSummary", 0);
				if (parseInt(hideSummary) === 1) {
					return true;
				}
			}
			return false;

		};

		$scope.isMessageApplicable = function(summaryTables) {
			//if both the summary tables (capacity usage & CAL usage) are empty, show corresponding message
			if (summaryTables && summaryTables.length > 0) {
				for (var i = 0; i < summaryTables.length; i++) {
					var table = summaryTables[i];
					if (table.tableData && table.tableData.length > 0) {
						return false;
					}
				}
			}
			$scope.summaryMsg = cvUtil.cvLocalize("NoDataAvailable");
			return true;
		};

		$scope.isErrorMsg = function(msg) {
			if (typeof (msg) !== "undefined") {
				if (msg.indexOf(cvUtil.cvLocalize("eSim_Sku_Applied")) >= 0) {
					return false;
				}
			}

			return true;

		};

		$scope.isPurchaseOrder = function() {
			if (typeof ($scope.useAsPO) !== "undefined") {
				if (parseInt($scope.useAsPO) === 1) {
					return false;
				}
			}

			return true;

		};

		$scope.isQuarterly = function() {
			var quarter = $scope.getParameter("quarter", 0);
			if (parseInt(quarter) === 1) {
				return true;
			}

			return false;

		};

		$scope.isSendReport = function() {
			var send = $scope.getParameter("send", "false");
			if (send.trim().toLowerCase() === "true") {
				return true;
			}

			return false;

		};

		$scope.isTotalRow = function(rowData) {
			if (rowData.cellValue.length > 0 && rowData.cellValue[0].value.toLowerCase() == 'sub total') {
				return true;
			}
			return false;
		};

		$scope.isFirstColumn = function(cellData) {
			if (cellData.value.match("^�")) {
				return true;
			}
			return false;
		};

		$scope.isAdditionalSection = function(section) {
			if (section.sectionTitle === "Additional Details") {
				return true;
			}
			return false;
		};

		$scope.isTenantBased = function(tenantSection) {
			if (tenantSection.detailTables[1].tableData[0].cellValue[1].value === "Tenant based report") {
				return true;
			} else {
				return false;
			}

		};

		$scope.hideVolumeDiscount = function(hasVolumeDiscount) {
			if (parseInt(hasVolumeDiscount) > 0) {
				return false;
			} else {
				return true;
			}
		};

		$scope.isEmptySection = function(section, dummySection) {
			//we just rename the section in this case
			if (section.sectionTitle === "Usage by CommCells") {
				if ($scope.isTenantBased(dummySection) === true) {
					section.sectionTitle = "Usage by Tenants";//rename usage by commcell section in case of tenant based contracts
				}
			}

			//hide capacity & client usage sections for CommCell based reports, hide client usage resultset for Tenant based reports
			if (section.sectionTitle === "License Exceptions") {
				if ($scope.isTenantBased(dummySection) === true) {
					section.sectionTitle = "Cost by Tenants";
					section.detailTables[1] = [];//we dont want the second dummy resultset to be displayed
					section.detailTables[0].title = "";//first resultset is having title 'Capacity Usage'. We have to remove this for tenant based report
					return false;
				} else {
					return true;
				}
			}
			//hide additional details section. This section will be shown separately at the end of the report after signature block
			if (section.sectionTitle === "Additional Details") {
				return true;
			}
			if (section.detailTables && section.detailTables.length > 0) {
				for (var i = 0; i < section.detailTables.length; i++) {
					var table = section.detailTables[i];
					if (table.tableData && table.tableData.length > 0) {
						return false;
					}
				}
			}

			return true;

		};

		$scope.isEmptyTable = function(data) {
			if (data.tableData && data.tableData.length > 0) {
				return false;
			}
			return true;

		};

		$scope.isStringColumn = function(cellData) {
			if (cellData.dataType == "string") {
				return true;
			}

			return false;
		};

		$scope.getParameter = function(name, defaultValue) {
			name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");

			var regexS = "[\\?&]" + name + "=([^&#]*)", regex = new RegExp(regexS), url = window.location.href;

			var results = regex.exec(url);
			if (results === null) {
				if (typeof defaultValue !== 'undefined') {
					return defaultValue;
				} else {
					return "";
				}
			} else {
				return decodeURIComponent(results[1].replace(/\+/g, " "));
			}
		};

		$scope.initialize();

	} ]);

})();