(function(exports) {
	'use strict';

	function initComponents(iAggregator) {
		loadContracts(false, iAggregator);
		initTable(iAggregator);
	}
	function loadContracts(isEdit, iAggregator) {
		cvUtil.loadPage("getContracts.do", {
			option : 7
		}, function(data) {
			var contractList = "";
			if (!isEdit) {
				contractList = $("#contractList");
			} else {
				contractList = $("#editContractList");
			}
			contractList.empty();
			if (data && data.aaData) {
				$.each(data.aaData, function(i, contract) {
					if (iAggregator !== 0) {
						if (contract[contractUtils.columns.CONTRACT] === contractUtils.global.LEGACY_CONTRACT_NAME) {
							contractList.append("<option value='" + contract[contractUtils.columns.CONTRACT_ID] + "'>" +
									contract[contractUtils.columns.CONTRACT] + "</option>");
						}
					} else {
						contractList.append("<option value='" + contract[contractUtils.columns.CONTRACT_ID] + "'>" +
								contract[contractUtils.columns.CONTRACT] + "</option>");
					}
				});
				if (data.aaData.length > 0 && isEdit === false) {
					loadNotAssociatedCustomers(iAggregator);
				}

			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Contracts"));

		}, {
			"async" : false
		});
	}
	function loadNotAssociatedCustomers(iAggregator) {

		cvUtil.loadPageWithSessionCheck("getNotAssociatedCustomer.do", {
			iAggregator : iAggregator
		}, function(data) {
			if (data && data.aaData) {
				if (iAggregator === 0) {
					$("#customerList").empty();
					var customerList = $("#customerList");
					$.each(data.aaData, function(i, customer) {
						customerList.append("<option value='" + customer[contractUtils.columns.GROUP_ID] + "'>" +
								customer[contractUtils.columns.CUSTOMER_NAME] + "</option>");
					});
				} else if (iAggregator === 1) {
					$("#aggregatorList").empty();
					var aggregatorList = $("#aggregatorList");
					$.each(data.aaData, function(i, aggregator) {
						aggregatorList.append("<option value='" + aggregator[contractUtils.columns.GROUP_ID] + "'>" +
								aggregator[contractUtils.columns.AGGREGATOR_NAME] + "</option>");
					});
				} else {
					$("#partnerList").empty();
					var partnerList = $("#partnerList");
					$.each(data.aaData, function(i, partner) {
						partnerList.append("<option value='" + partner[contractUtils.columns.GROUP_ID] + "'>" +
								partner[contractUtils.columns.PARTNER_NAME] + "</option>");
					});
				}
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Customers"));

		});
	}

	function initTable(iAggregator) {
		var view = {};
		view.title = '';
		view.filters = true;
		view.collapsible = false;
		view.alarm = false;
		view.exportExcel = true;
		view.colChooser = true;
		if (iAggregator === 0) {
			$("#customerTable").empty();
			$('#customerTableWrapper').append(SurveyTemplates.makeReportTable('customerTable', view));
		} else if (iAggregator === 1) {
			$("#aggregatorTable").empty();
			$('#aggregatorTableWrapper').append(SurveyTemplates.makeReportTable('aggregatorTable', view));
		} else {
			$("#partnerTable").empty();
			$('#partnerTableWrapper').append(SurveyTemplates.makeReportTable('partnerTable', view));
		}
		getContractAssociations(iAggregator);
	}

	function getContractAssociations(iAggregator) {
		cvUtil.loadPageWithSessionCheck("getContractAssociations.do", {
			iAggregator : iAggregator
		}, function(data) {
			contractUtils.addActionButtonToTableRows(data, true);
			if (iAggregator === 0) {
				Reports.configureClientSideTable('customerTable', '', data);
				$("#customerTable.reportTable table th:last-child").css("width", "80px");
				//hide "delete" & "edit" buttons from users with "view" permission
				if (securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW) {
					$("#customerTableWrapper .icon-trash").hide();
					$("#customerTableWrapper .icon-edit").hide();
				}
			} else if (iAggregator === 1) {
				Reports.configureClientSideTable('aggregatorTable', '', data);
				$("#aggregatorTable.reportTable table th:last-child").css("width", "80px");
				if (securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW) {
					$("#aggregatorTableWrapper .icon-trash").hide();
					$("#aggregatorTableWrapper .icon-edit").hide();
				}
			} else {
				Reports.configureClientSideTable('partnerTable', '', data);
				$("#partnerTable.reportTable table th:last-child").css("width", "80px");
				if (securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW) {
					$("#partnerTableWrapper .icon-trash").hide();
					$("#partnerTableWrapper .icon-edit").hide();
				}
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Customer_Associations"));

		});
	}

	function addAssociation(contractId, contractName, customerId, customerName, minFee, billingType, useAsPO,
			iAggregator) {
		cvUtil.loadPageWithSessionCheck("addAssociation.do", {
			contractId : contractId,
			customerId : customerId,
			minimumFee : minFee,
			billingType : billingType,
			useAsPO : useAsPO,
			iAggregator : iAggregator
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(cvUtil.cvLocalize("Add_Association_Success"));
				initTable(iAggregator);
				if (iAggregator === 0) {
					$("#customerList option:selected").remove();
				} else if (iAggregator === 1) {
					$("#aggregatorList option:selected").remove();
				} else {
					$("#partnerList option:selected").remove();
				}
			} else {
				cvUtil.errorToast(cvUtil.cvLocalize("Add_Association_Failure"));
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Add_Association_Failure"));

		});
	}

	function initAssociationDialog(rowData, iAggregator) {
		loadContracts(true, iAggregator);
		clearComponents();
		if (rowData) {
			if (iAggregator === 0) {
				$("#editCustomerList").val(rowData[contractUtils.columns.CUSTOMER_NAME]);
				$("#divGrpLabel").text(contractUtils.columns.CUSTOMER_NAME);
				$("#editMinFee").val("0");
			} else if (iAggregator === 1) {
				$("#editCustomerList").val(rowData[contractUtils.columns.AGGREGATOR_NAME]);
				$("#divGrpLabel").text(contractUtils.columns.AGGREGATOR_NAME);
				$("#editMinFee").val(rowData[contractUtils.columns.MINIMUM_FEE]);
			} else {
				$("#editCustomerList").val(rowData[contractUtils.columns.PARTNER_NAME]);
				$("#divGrpLabel").text(contractUtils.columns.PARTNER_NAME);
				$("#editContractList").attr("disabled", "disabled");
				$("#editMinFee").val(rowData[contractUtils.columns.DISCOUNT_PERCENTAGE]);
			}
			$("#editContractList option[value='" + rowData[contractUtils.columns.CONTRACT_ID] + "']").attr("selected",
					"selected");
			var billType = 0;
			if (rowData[contractUtils.columns.BILLING_TYPE] === cvUtil.cvLocalize("Quarterly")) {
				billType = 1;
			}
			$("#editBillingTypeList option[value='" + billType + "']").attr("selected", "selected");
			var isPO = 0;
			if (rowData[contractUtils.columns.USE_AS_PO] === cvUtil.cvLocalize("No")) {
				isPO = 1;
			}
			$("input[name=editUseAsPOType][value=" + isPO + "]").prop("checked", true);
		}
	}

	function clearComponents() {
		$("#editCommcellGroup").val("");
		$("#editMinFee").val("");
		$("#editContractList").find('option:selected').removeAttr("selected");
		$("#editBillingTypeList").find('option:selected').removeAttr("selected");
		$("input[name=editUseAsPOType][value=0]").prop("checked", false);
		$("input[name=editUseAsPOType][value=1]").prop("checked", false);
	}

	function modifyAssociation(contractId, contractName, customerId, customerName, minFee, billingType, useAsPO,
			iAggregator, discount) {

		cvUtil.loadPageWithSessionCheck("modifyAssociation.do", {
			contractId : contractId,
			customerId : customerId,
			minimumFee : minFee,
			billingType : billingType,
			useAsPO : useAsPO,
			iAggregator : iAggregator,
			discount : JSON.stringify(discount)
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(cvUtil.cvLocalize("Update_Association_Success"));
				initTable(iAggregator);
			} else {
				cvUtil.errorToast(cvUtil.cvLocalize("Update_Association_Failure"));
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Update_Association_Failure"));

		});
	}

	function deleteAssociation(customerId, iAggregator) {
		cvUtil.loadPageWithSessionCheck("deleteAssociation.do", {
			groupId : customerId,
			iAggregator : iAggregator
		}, function(data) {
			if (data && data.errorCode === 0) {
				$("#delete-contract-entity-dialog-form").dialog("close");
				$("#delete-contract-entity-dialog-form").dialog("destroy");
				cvUtil.toast(cvUtil.cvLocalize("Delete_Association_Success"));
				initTable(iAggregator);
				loadNotAssociatedCustomers(iAggregator);
			} else {
				cvUtil.errorToast(cvUtil.cvLocalize("Delete_Association_Failure"));
			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Delete_Association_Failure"));

		});
	}

	exports.contractAssociation = {
		init : initComponents,
		loadContracts : loadContracts,
		loadNotAssociatedCustomers : loadNotAssociatedCustomers,
		initAssociationDialog : initAssociationDialog,
		addAssociation : addAssociation,
		modifyAssociation : modifyAssociation,
		deleteAssociation : deleteAssociation,
		loadContractAssociations : getContractAssociations
	};

})(window);