(function(detailExports) {
	'use strict';

	var dialogData;
	var editingRow = false;
	var rowIndex = -1;
	var contractData = {};
	var emptyContract = false;
	var detailDialogData;
	var isEditButtonClicked = false;
	var rowDataToEdit;

	function initDialog(isEdit, rowData) {
		if (rowData) {
			contractData.contractId = rowData[contractUtils.columns.CONTRACT_ID];
		}
		isEditButtonClicked = isEdit;
		rowDataToEdit = rowData;

		clearContractDetailFields(false);
		initCurrencyList();
		//initCommcellGroupList();
		//initTenantList("0");
		initComponents(rowData);
		initBillingGroupList();
		initDetailTable(isEdit, rowData, 0);
	}

	function initCurrencyList() {
		cvUtil.loadPage("getCurrencies.do", {
			option : 11
		}, function(data) {
			var currencyList = $("#contractCurrencyList");
			currencyList.empty();
			$.each(data.aaData, function(index, currency) {
				currencyList.append("<option value='" + currency[contractUtils.columns.CURRENCY] + "'>" +
						currency[contractUtils.columns.CURRENCY] + "</option>");
			});

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Currency_List"));

		}, {
			"async" : false
		});
	}

	function initCommcellGroupList() {
		cvUtil.loadPage("getCommCellGroupList.do", {
			iAggregator : 3
		}, function(data) {
			var ccGroupList = $("#ccGroupList");
			ccGroupList.empty();
			$.each(data.aaData, function(index, ccGroup) {
				ccGroupList.append("<option value='" + ccGroup[contractUtils.columns.COMMCELL_GROUP_ID] + "'>" +
						ccGroup[contractUtils.columns.COMMCELL_GROUP_NAME] + "</option>");
			});
			if (!editingRow || emptyContract) {
				$("#ccGroupList").val($("#ccGroupList option:first").val());
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Billing_Group_Details"));
		}, {
			"async" : false
		});
	}

	function initTenantList(groupId) {
		cvUtil.loadPage("getTenants.do", {
			ccGrpId : groupId
		}, function(data) {
			var tenantList = $("#tenantList");
			tenantList.empty();
			$.each(data.aaData, function(index, tenant) {
				tenantList.append("<option value='" + tenant[contractUtils.columns.TENANT_ID] + "'>" +
						tenant[contractUtils.columns.TENANT] + "</option>");
			});

			if (!editingRow || emptyContract) {
				$("#tenantList").val($("#tenantList option:first").val());
			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Tenant_List"));

		}, {
			"async" : false
		});
	}

	$("#ccGroupList").change(function() {
		var groupId = $("#ccGroupList option:selected").val();
		initTenantList(groupId);
	});

	function initContractDetailFields(rowData) {
		dialogData = rowData;

		if (rowData === undefined || rowData === null) {
			setLabels(true); //default to Capacity
		} else {
			setLabels(rowData[contractUtils.columns.COUNTING_TYPE] === 'Capacity');
		}

		if (rowData === undefined || rowData === null) {
			$("#addContractDetailBtn").val(cvUtil.cvLocalize("Add_Price"));
			editingRow = false;
			return;
		}

		if (rowData[contractUtils.columns.CONTRACT]) {
			$("#contractName").val(rowData[contractUtils.columns.CONTRACT]);
		}

		if (rowData[contractUtils.columns.CONTRACT_DESCRIPTION]) {
			$("#contractDesc").val(rowData[contractUtils.columns.CONTRACT_DESCRIPTION]);
		}

		if (rowData[contractUtils.columns.CURRENCY]) {
			$("#contractCurrencyList option[value='" + rowData[contractUtils.columns.CURRENCY] + "']").attr("selected",
					"selected");
		}

		if (rowData[contractUtils.columns.CONTRACT_TYPE]) {
			var contractType = 0;
			if (rowData[contractUtils.columns.CONTRACT_TYPE].toLowerCase() === contractUtils.global.DEFAULT_CONTRACT_TYPE
					.toLowerCase()) {
				contractType = 0;
			} else if (rowData[contractUtils.columns.CONTRACT_TYPE].toLowerCase() === contractUtils.global.TENANT_CONTRACT_TYPE
					.toLowerCase()) {
				contractType = 1;
				initCommcellGroupList();
				initTenantList("0");
			}
			$("#contractType option[value='" + contractType + "']").attr("selected", "selected");
		}

		if (rowData[contractUtils.columns.BILLING_GROUP_ID] === 0) {// no billing group defined
			setLabels(true); //default to Capacity
			emptyContract = true;
		} else {
			$("#contract-dialog-form #countingType").val(rowData[contractUtils.columns.COUNTING_TYPE]);
			$("#billingGroupList").val(rowData[contractUtils.columns.BILLING_GROUP_ID]);
			$("#price").val(rowData[contractUtils.columns.UNIT_PRICE]);
			$("#initialQuantity").val(rowData[contractUtils.columns.BASELINE]);
			$("#upperLimit").val(rowData[contractUtils.columns.UPPER_LIMIT]);
			$("#lowerLimit").val(rowData[contractUtils.columns.LOWER_LIMIT]);
			$("#tenantList").val(rowData[contractUtils.columns.TENANT_ID]);
			$("#addContractDetailBtn").val(cvUtil.cvLocalize("Update_Price"));
			$("#clearContractDetailBtn").val(cvUtil.cvLocalize("Add_New"));
			$("#billingGroupList").attr("disabled", true);
			$("#billingGroupList").addClass("disabled");
			$("#tenantList").attr("disabled", true);
			$("#tenantList").addClass("disabled");
			$("#IncludeInVDList").val(rowData[contractUtils.columns.INCLUDE_IN_VOLUME_DISCOUNT]);
		}

		editingRow = true;
	}

	function initComponents(rowData) {
		$("#contractName").val("");
		$("#contractDesc").val("");
		$("#contractCurrencyList option[value='" + contractUtils.global.DEFAULT_CURRENCY + "']").attr("selected",
				"selected");
		$("#contractType option[value='0']").attr("selected", "selected");
		$("#contractName").removeClass("errorInput");
		$("#errorMessage").html("");
		$("#errorMessage").removeClass("invalidInput");
		initContractDetailFields(rowData);

	}

	function setLabels(isCapacity) {
		if (isCapacity) {
			$("#initialQuantityLabel").html(contractUtils.labels.BASELINE_TB);
			$("#lowerLimitLabel").html(contractUtils.labels.LOWER_LIMIT_TB);
			$("#upperLimitLabel").html(contractUtils.labels.UPPER_LIMIT_TB);
		} else {
			$("#initialQuantityLabel").html(contractUtils.labels.BASELINE);
			$("#lowerLimitLabel").html(contractUtils.labels.LOWER_LIMIT);
			$("#upperLimitLabel").html(contractUtils.labels.UPPER_LIMIT);
		}
	}

	function initDetailTable(isEdit, rowData, isTypeChange) {

		if (!isEdit) {
			var view = {};
			view.title = '';
			$("#contractDetailTable").empty();
			$('#contractDetailTableWrapper').append(SurveyTemplates.makeReportTable('contractDetailTable', view));

			var contractDetailData = getEmptyTableData();
			contractUtils.addActionButtonToTableRows(contractDetailData);
			contractUtils.addRenderers(contractDetailData);
			contractUtils.handleTenantColumn(contractDetailData, parseInt($("#contractType").val()));
			Reports.configureClientSideTable("contractDetailTable", '', contractDetailData);
			$("#contractDetailTableWrapper .icon-billing").hide();
			$("#contractDetailTableWrapper .icon-send").hide();
		} else {
			if (isTypeChange === 0) {
				cvUtil.loadPage("getContract.do", {
					contractId : rowData[contractUtils.columns.CONTRACT_ID]
				}, function(data) {
					loadContractDetail(data, rowData);
				}, function(xhr, ajaxOptions, thrownError) {
					cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Contract_Details"));

				});
			} else {
				loadContractDetail(detailDialogData, rowData);
			}
		}

	}

	function loadContractDetail(data, rowData) {
		var view = {};
		view.title = '';
		$("#contractDetailTable").empty();
		$('#contractDetailTableWrapper').append(SurveyTemplates.makeReportTable('contractDetailTable', view));

		var contractDetailData = data;
		contractUtils.addActionButtonToTableRows(contractDetailData);
		contractUtils.addRenderers(contractDetailData, true);
		contractUtils.handleTenantColumn(contractDetailData, parseInt($("#contractType").val()));

		var colReorder = {
			allowReorder : false
		};
		Reports.configureClientSideTable("contractDetailTable", '', contractDetailData, cvUtil
				.cvLocalize("NoDataAvailable"), {
			oColReorder : colReorder
		});
		$("#contractDetailTableWrapper .icon-billing").hide();
		$("#contractDetailTableWrapper .icon-send").hide();
		$.each(data.aaData, function(i, dat) {
			if (i === 0) {
				if (dat[contractUtils.columns.BILLING_GROUP_ID] === rowData[contractUtils.columns.BILLING_GROUP_ID]) {
					initContractDetailFields(dat);
					contractUtils.global.ROW_INDEX = i;
				}
			}
		});
		detailDialogData = data;

	}

	function initBillingGroupList() {
		cvUtil.loadPage("getBillingGroups.do", {
			option : 8
		}, function(data) {
			var listEle = $("#billingGroupList");
			listEle.empty();
			$.each(data.aaData, function(index, group) {
				listEle.append("<option value='" + group[contractUtils.columns.BILLING_GROUP_ID] +
						"' data-counting-type='" + group[contractUtils.columns.COUNTING_TYPE] + "'" +
						" data-entity-type='" + group[contractUtils.columns.ENTITY_TYPE] + "'>" +
						group[contractUtils.columns.BILLING_GROUP] + "</option>");
			});

			if (!editingRow || emptyContract) {
				$("#billingGroupList").val($("#billingGroupList option:first").val());
				$("#contract-dialog-form #countingType").val($("#billingGroupList option:selected")
						.attr("data-counting-type"));
			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Billing_Group_List"));

		});
	}

	function initLicenses() {
		cvUtil.loadPage("getLicenseTypes.do", '', function(data) {
			var listEle = $("#licenseList");
			$.each(data.aaData, function(index, lic) {
				listEle.append("<option value='" + lic.id + "'>" + lic.name + "</option>");
			});

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Licenses"));

		});

		cvUtil.loadPage("getAgentTypes.do", '', function(data) {
			var listEle = $("#agentList");
			$.each(data.aaData, function(index, lic) {
				listEle.append("<option value='" + lic.id + "'>" + lic.name + "</option>");
			});

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Agents"));

		});
	}

	function clearContractDetailFields(isNew) {
		$("#addContractDetailBtn").val(cvUtil.cvLocalize("Add_Price"));
		$("#clearContractDetailBtn").val(cvUtil.cvLocalize("Clear"));
		$("#billingGroupList").attr("disabled", false);
		$("#billingGroupList").removeClass("disabled");
		$("#tenantList").attr("disabled", false);
		$("#tenantList").removeClass("disabled");
		clearPriceFields();
		if (isNew) {
			autofillPriceFields();
		}
		editingRow = false;
	}

	function autofillPriceFields() {

		var table = $("#contractDetailTable_table").dataTable(), rows = table.fnGetData();

		//auto fill baseline field if already same billing group is present in the price group list
		var exist = false;
		var i = 0;
		if ($("#contractType option:selected").text() === contractUtils.global.DEFAULT_CONTRACT_TYPE) {
			for (i = 0; i < rows.length; i++) {
				if (rows[i][contractUtils.columns.BILLING_GROUP] === $("#billingGroupList option:selected").text()) {
					$("#initialQuantity").val(rows[i][contractUtils.columns.BASELINE]);
					exist = true;
					break;
				}
			}
		} else if ($("#contractType option:selected").text() === contractUtils.global.TENANT_CONTRACT_TYPE) {
			for (i = 0; i < rows.length; i++) {
				if (rows[i][contractUtils.columns.BILLING_GROUP] === $("#billingGroupList option:selected").text() &&
						rows[i][contractUtils.columns.TENANT_ID] === $("#tenantList option:selected").val()) {
					$("#initialQuantity").val(rows[i][contractUtils.columns.BASELINE]);
					exist = true;
					break;
				}
			}
		}
		if (!exist) {
			$("#initialQuantity").val("");
		}
	}

	function clearPriceFields() {
		$("#price").val("");
		$("#initialQuantity").val("");
		$("#upperLimit").val("");
		$("#lowerLimit").val("");
		$("#errorMessage").removeClass("invalidInput");
		$("#errorMessage").html("");
		$("#IncludeInVDList").val("No");
	}

	function getEmptyTableData() {
		var columns = [], contractDetailData = {}, i, col, cols = [
				contractUtils.columns.TENANT,
				contractUtils.columns.TENANT_ID,
				contractUtils.columns.BILLING_GROUP,
				contractUtils.columns.BILLING_GROUP_ID,
				contractUtils.columns.COUNTING_TYPE,
				contractUtils.columns.COUNTING_TYPE_ID,
				contractUtils.columns.BASELINE,
				contractUtils.columns.UNIT_PRICE,
				contractUtils.columns.LOWER_LIMIT,
				contractUtils.columns.UPPER_LIMIT,
				contractUtils.columns.INCLUDE_IN_VOLUME_DISCOUNT,
				contractUtils.columns.UNIQUE_ID ];

		for (i = 0; i < cols.length; i++) {
			col = {
				bFilterRendered : true,
				bHidden : false,
				bHiddenColumnSort : false,
				bMandatoryCol : false,
				bSortable : true,
				bUseRendered : false,
				bVisible : true,
				doNotShow : false,
				mDataProp : cols[i],
				originalColName : cols[i],
				sName : cols[i],
				sTitle : cols[i],
				sqlType : "String"
			};

			if (cols[i].indexOf("DONOTSHOW") !== -1 || cols[i].indexOf("HIDDEN") !== -1) {
				col.doNotShow = true;
				col.bVisible = false;
				col.bHidden = true;
			}
			if ($("#contractType").val() === "0") {//for CommCell based contracts, do not show Tenant column
				if (cols[i].indexOf(contractUtils.columns.TENANT) !== -1) {
					col.doNotShow = true;
					col.bVisible = false;
					col.bHidden = true;
				}
			}
			columns.push(col);
		}

		contractDetailData.aaData = {};
		contractDetailData.aoColumns = columns;
		contractDetailData.iDisplayLength = 0;
		contractDetailData.iTotalDisplayRecords = 0;
		contractDetailData.iTotalRecords = 0;
		contractDetailData.offset = 0;

		return contractDetailData;

	}

	function isValidInput(tableData) {
		//check if contract name is valid
		if ($("#contractName").val().trim() === "") {
			$("#errorMessage").html(cvUtil.cvLocalize("Invalid_Contract_Name"));
			$("#errorMessage").addClass("invalidInput");
			$("#contractName").focus();
			$("#contractName").addClass("errorInput");

			return false;
		}

		var groups = {}, billingGroups = [], i, j, flag, flag1;
		for (i = 0; i < tableData.length; i++) {
			flag = false;
			for (j = 0; j < billingGroups.length; j++) {
				if (parseInt(tableData[i][contractUtils.columns.BILLING_GROUP_ID]) === billingGroups[j].billingGroupId &&
						tableData[i][contractUtils.columns.TENANT_ID] === billingGroups[j].tenantId) {
					flag = true;
					break;
				}
			}
			if (flag === false) {
				groups = {
					"tenantId" : tableData[i][contractUtils.columns.TENANT_ID],
					"billingGroupId" : parseInt(tableData[i][contractUtils.columns.BILLING_GROUP_ID])
				};
				billingGroups.push(groups);
			}
		}
		for (i = 0; i < tableData.length; i++) {
			//check if duplicate rows with same lower limits or same upper limits are present for the same billing group within same tenant
			for (j = i + 1; j < tableData.length; j++) {
				if (parseInt(tableData[i][contractUtils.columns.BILLING_GROUP_ID]) === parseInt(tableData[j][contractUtils.columns.BILLING_GROUP_ID]) &&
						parseInt(tableData[i][contractUtils.columns.TENANT_ID]) === parseInt(tableData[j][contractUtils.columns.TENANT_ID])) {
					if (parseInt(tableData[i][contractUtils.columns.LOWER_LIMIT]) === parseInt(tableData[j][contractUtils.columns.LOWER_LIMIT]) ||
							parseInt(tableData[i][contractUtils.columns.UPPER_LIMIT]) === parseInt(tableData[j][contractUtils.columns.UPPER_LIMIT])) {

						$("#errorMessage").html(cvUtil.cvLocalize("Duplicate_Price_Tier"));
						$("#errorMessage").addClass("invalidInput");

						return false;
					}
				}
			}
			//check if upper limit is greater than lower limit for all rows
			if (parseInt(tableData[i][contractUtils.columns.UPPER_LIMIT]) > -1) {
				if (parseInt(tableData[i][contractUtils.columns.LOWER_LIMIT]) >= parseInt(tableData[i][contractUtils.columns.UPPER_LIMIT])) {

					$("#errorMessage").html(cvUtil.cvLocalize("Invalid_Upper_Limit"));
					$("#errorMessage").addClass("invalidInput");

					return false;
				}
			}
		}
		//check if LowerLimit=0 & UpperLimit=-1 are present for all billing groups within same tenant
		for (i = 0; i < billingGroups.length; i++) {
			flag = false;
			for (j = 0; j < tableData.length; j++) {
				if (billingGroups[i].billingGroupId === parseInt(tableData[j][contractUtils.columns.BILLING_GROUP_ID]) &&
						billingGroups[i].tenantId === tableData[j][contractUtils.columns.TENANT_ID] &&
						parseInt(tableData[j][contractUtils.columns.LOWER_LIMIT]) === 0) {
					flag = true;
					break;
				}
			}
			flag1 = false;
			for (j = 0; j < tableData.length; j++) {
				if (billingGroups[i].billingGroupId === parseInt(tableData[j][contractUtils.columns.BILLING_GROUP_ID]) &&
						billingGroups[i].tenantId === tableData[j][contractUtils.columns.TENANT_ID] &&
						parseInt(tableData[j][contractUtils.columns.UPPER_LIMIT]) === -1) {
					flag1 = true;
					break;
				}
			}
			if (flag === false && flag1 === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Price_Tier"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			} else if (flag === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Lower_Limit"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			} else if (flag1 === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Upper_Limit"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			}
		}
		//check for continuity of price tiers within same tenant
		for (i = 0; i < billingGroups.length; i++) {
			for (j = 0; j < tableData.length; j++) {
				if (billingGroups[i].billingGroupId === parseInt(tableData[j][contractUtils.columns.BILLING_GROUP_ID]) &&
						billingGroups[i].tenantId === tableData[j][contractUtils.columns.TENANT_ID]) {
					flag = false;
					for (var k = 0; k < tableData.length; k++) {
						if (billingGroups[i].billingGroupId === parseInt(tableData[k][contractUtils.columns.BILLING_GROUP_ID]) &&
								billingGroups[i].tenantId === tableData[k][contractUtils.columns.TENANT_ID]) {
							if (parseInt(tableData[j][contractUtils.columns.UPPER_LIMIT]) <= 0 ||
									parseInt(tableData[j][contractUtils.columns.UPPER_LIMIT]) === parseInt(tableData[k][contractUtils.columns.LOWER_LIMIT])) {
								flag = true;
								break;
							}
						}
					}
					flag1 = false;
					for (k = 0; k < tableData.length; k++) {
						if (billingGroups[i].billingGroupId === parseInt(tableData[k][contractUtils.columns.BILLING_GROUP_ID]) &&
								billingGroups[i].tenantId === tableData[k][contractUtils.columns.TENANT_ID]) {
							if (parseInt(tableData[j][contractUtils.columns.LOWER_LIMIT]) === 0 ||
									parseInt(tableData[j][contractUtils.columns.LOWER_LIMIT]) === parseInt(tableData[k][contractUtils.columns.UPPER_LIMIT])) {
								flag1 = true;
								break;
							}
						}
					}
					if (flag === false || flag1 === false) {
						$("#errorMessage").html(cvUtil.cvLocalize("Invalid_Price_Tier"));
						$("#errorMessage").addClass("invalidInput");

						return false;
					}
				}
			}
		}
		//it is mandatory to associate all billing groups to default tenant
		for (i = 0; i < billingGroups.length; i++) {
			flag = false;
			for (j = 0; j < billingGroups.length; j++) {
				if (billingGroups[i].billingGroupId === billingGroups[j].billingGroupId) {
					if (billingGroups[j].tenantId === "-1") {
						flag = true;
						break;
					}
				}
			}
			if (flag === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Default_Tenant_Not_Present"));
				$("#errorMessage").addClass("invalidInput");
				return false;
			}
		}

		return true;
	}

	function saveContract(option, isDelete, callbacks) {

		var contract = {}, billingGroups = [], group, table, tableData, ctr;

		contract.contractName = $("#contractName").val();
		contract.description = $("#contractDesc").val();
		contract.currency = $("#contractCurrencyList option:selected").text();
		contract.contractType = $("#contractType option:selected").val();
		if (option === 2) {
			contract.contractId = contractData.contractId;
		}

		contract.billingGroup = billingGroups;
		table = $("#contractDetailTable_table").dataTable();
		tableData = table.fnGetData();

		if (!isValidInput(tableData)) {
			return;
		}

		for (ctr = 0; ctr < tableData.length; ctr++) {
			group = {};
			group.billingGroupId = tableData[ctr][contractUtils.columns.BILLING_GROUP_ID];
			group.billingGroupName = tableData[ctr][contractUtils.columns.BILLING_GROUP];
			group.price = tableData[ctr][contractUtils.columns.UNIT_PRICE] || 0;
			group.baseLine = tableData[ctr][contractUtils.columns.BASELINE] || 0;
			group.lowerLimit = tableData[ctr][contractUtils.columns.LOWER_LIMIT] || 0;
			group.upperLimit = tableData[ctr][contractUtils.columns.UPPER_LIMIT] || 0;
			group.uniqueId = tableData[ctr][contractUtils.columns.UNIQUE_ID] || 0;
			group.tenantId = tableData[ctr][contractUtils.columns.TENANT_ID] || '-1';
			group.tenantName = tableData[ctr][contractUtils.columns.TENANT] || '';
			if (tableData[ctr][contractUtils.columns.INCLUDE_IN_VOLUME_DISCOUNT] === "Yes"){group.includeInVolumeDiscount = 1;} else {group.includeInVolumeDiscount = 0;}

			billingGroups.push(group);
		}

		cvUtil.loadPageWithPOST("saveContract.do", {
			contract : JSON.stringify(contract),
			option : option,
			isDelete : isDelete
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(data.message);
				if (callbacks) {
					$.each(callbacks, function(i, cb) {
						cb.call();
					});
				}
			} else {
				cvUtil.errorToast(data.message);
			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Save_Contract_Failure"));

		});
	}

	function getColumnIndex(table, columnName) {
		var columns = table.fnSettings().aoColumns, i = 0;

		for (i = 0; i < columns.length; i++) {
			if (columns[i].mDataProp === columnName) {
				return i;
			}
		}

		return -1;
	}

	function addContractDetail() {
		var table = $("#contractDetailTable_table").dataTable(), groupName = "", groupId = 0, rows = table.fnGetData(), columnIndex = -1, baselineIndex = [];

		//unit price > 0 is mandatory for all billing groups
		if ($("#price").val().trim() !== "" && parseFloat($("#price").val()) > 0) {
			$("#errorMessage").removeClass("invalidInput");
			$("#errorMessage").html("");
			if (editingRow && rows.length > 0) { // check rows length to handle case where there is no billing group
				var rowIndex = -1;

				rowIndex = contractUtils.global.ROW_INDEX;

				/*
				 * if (dialogData) { for (var i=0; i< rows.length; i++) { if
				 * (rows[i][contractUtils.columns.BILLING_GROUP_ID] ==
				 * dialogData[contractUtils.columns.BILLING_GROUP_ID]) { rowIndex = i; break; } } }
				 */

				if (rowIndex > -1) {
					//columnIndex = getColumnIndex(table, contractUtils.columns.BASELINE);
					//table.fnUpdate($("#initialQuantity").val(), baselineIndex[j], columnIndex);

					columnIndex = getColumnIndex(table, contractUtils.columns.UNIT_PRICE);
					table.fnUpdate($("#price").val(), rowIndex, columnIndex);

					columnIndex = getColumnIndex(table, contractUtils.columns.UPPER_LIMIT);
					if ($("#upperLimit").val().trim() === "" || $("#upperLimit").val().trim() === "0") {
						table.fnUpdate("-1", rowIndex, columnIndex);
					} else {
						table.fnUpdate($("#upperLimit").val(), rowIndex, columnIndex);
					}

					columnIndex = getColumnIndex(table, contractUtils.columns.LOWER_LIMIT);
					if ($("#lowerLimit").val().trim() === "") {
						table.fnUpdate("0", rowIndex, columnIndex);
					} else {
						table.fnUpdate($("#lowerLimit").val(), rowIndex, columnIndex);
					}

					columnIndex = getColumnIndex(table, contractUtils.columns.TENANT_ID);
					var colIdx = getColumnIndex(table, contractUtils.columns.TENANT);
					if ($("#tenantList option:selected").val() === undefined ||
							$("#tenantList option:selected").val().trim() === "") {
						table.fnUpdate("-1", rowIndex, columnIndex);
						table.fnUpdate("(default)", rowIndex, colIdx);
					} else {
						table.fnUpdate($("#tenantList option:selected").val(), rowIndex, columnIndex);
						table.fnUpdate($("#tenantList option:selected").text(), rowIndex, colIdx);
					}

					columnIndex = getColumnIndex(table, contractUtils.columns.INCLUDE_IN_VOLUME_DISCOUNT);
					table.fnUpdate($("#IncludeInVDList option:selected").text(), rowIndex, columnIndex);
				}
			} else {
				var row = {};
				row[contractUtils.columns.BILLING_GROUP] = $("#billingGroupList option:selected").text();
				row[contractUtils.columns.BILLING_GROUP_ID] = $("#billingGroupList").val();
				row[contractUtils.columns.UNIT_PRICE] = $("#price").val();
				if ($("#upperLimit").val().trim() === "" || $("#upperLimit").val().trim() === "0") {
					row[contractUtils.columns.UPPER_LIMIT] = "-1";
				} else {
					row[contractUtils.columns.UPPER_LIMIT] = $("#upperLimit").val();
				}
				if ($("#lowerLimit").val().trim() === "") {
					row[contractUtils.columns.LOWER_LIMIT] = "0";
				} else {
					row[contractUtils.columns.LOWER_LIMIT] = $("#lowerLimit").val();
				}
				if ($("#initialQuantity").val().trim() === "") {
					row[contractUtils.columns.BASELINE] = "0";
				} else {
					row[contractUtils.columns.BASELINE] = $("#initialQuantity").val();
				}
				if ($("#tenantList option:selected").val() === undefined) {
	                row[contractUtils.columns.TENANT_ID] = "-1";
	                row[contractUtils.columns.TENANT] = "(default)";
	            } else {
				    row[contractUtils.columns.TENANT_ID] = $("#tenantList option:selected").val();
				    row[contractUtils.columns.TENANT] = $("#tenantList option:selected").text();
                }
				row[contractUtils.columns.ENTITY_TYPE_ID] = 0;
				row[contractUtils.columns.COUNTING_TYPE] = $("#countingType").val();
				row[contractUtils.columns.COUNTING_TYPE_ID] = 0;
				row[contractUtils.columns.INCLUDE_IN_VOLUME_DISCOUNT] = $("#IncludeInVDList option:selected").val();
				row["ACTION_COL"] = "";

				if (dialogData) {
					row[contractUtils.columns.CONTRACT] = dialogData[contractUtils.columns.CONTRACT];
					row[contractUtils.columns.CONTRACT_ID] = dialogData[contractUtils.columns.CONTRACT_ID];
				} else {
					row[contractUtils.columns.CONTRACT] = $("#contractName").val();
					row[contractUtils.columns.CONTRACT_DESCRIPTION] = $("#contractDesc").val();
					row[contractUtils.columns.CONTRACT_CURRENCY] = $("#contractCurrencyList option:selected").text();
					row[contractUtils.columns.CONTRACT_TYPE] = $("#contractType option:selected").text();

				}

				row[contractUtils.columns.COUNTING_TYPE] = $("#contract-dialog-form #countingType").val();
				//		      row[contractUtils.columns.ENTITY_TYPE]		=   $("#contract-dialog-form #entityType").val();
				row[contractUtils.columns.UNIQUE_ID] = 0;

				table.fnAddData(row);
				table.fnSettings()._iDisplayLength = table.fnGetData().length;
				table.fnDraw();
			}

			var i = 0, j = 0;
			//baseline should be updated for all rows of that same billing group
			//For this, find the rows to be updated
			if (dialogData) {
				if ($("#contractType option:selected").text() === contractUtils.global.DEFAULT_CONTRACT_TYPE) {
					for (i = 0; i < rows.length; i++) {
						if (rows[i][contractUtils.columns.BILLING_GROUP] === $("#billingGroupList option:selected")
								.text()) {
							baselineIndex[j++] = i;
						}
					}
				} else if ($("#contractType option:selected").text() === contractUtils.global.TENANT_CONTRACT_TYPE) {//for tenant contracts, tenant and billing group should match for updating baseline
					for (i = 0; i < rows.length; i++) {
						if (rows[i][contractUtils.columns.BILLING_GROUP] === $("#billingGroupList option:selected")
								.text() &&
								rows[i][contractUtils.columns.TENANT_ID] === $("#tenantList option:selected").val()) {
							baselineIndex[j++] = i;
						}
					}
				}
			}
			//update all the above rows in the table with new baseline
			columnIndex = getColumnIndex(table, contractUtils.columns.BASELINE);
			for (j = 0; j < baselineIndex.length; j++) {
				if ($("#initialQuantity").val().trim() === "") {
					table.fnUpdate("0", baselineIndex[j], columnIndex);
				} else {
					table.fnUpdate($("#initialQuantity").val(), baselineIndex[j], columnIndex);
				}
			}

			$("#contractDetailTableWrapper .icon-billing").hide();
			$("#contractDetailTableWrapper .icon-send").hide();

			//		  clearPriceFields();
			clearContractDetailFields(true);
		} else {
			$("#errorMessage").addClass("invalidInput");
			$("#errorMessage").html(cvUtil.cvLocalize("Non_Zero_Unit_Price"));
			$("#price").focus();
		}
	}

	function deleteContractDetail(row) {
		var table = $("#contractDetailTable_table").dataTable();
		var index = table.fnGetPosition(row.find('td')[0])[0];
		table.fnDeleteRow(index);
		// clearPriceFields();
		clearContractDetailFields(true);
	}

	function getLicenseCount() {
		var table = $("#contractDetailTable_table").dataTable();
		return table.fnGetData().length;
	}

	$("#contractType").change(function() {
		if (parseInt($("#contractType").val()) > 0) {
			initCommcellGroupList();
			initTenantList("0");
		}
		initDetailTable(isEditButtonClicked, rowDataToEdit, 1);
		$("#errorMessage").removeClass("invalidInput");
		$("#errorMessage").html("");
	});

	detailExports.contractDetailDialog = {
		//			openDialog: openContractDetailDialog,
		initDialog : initDialog,
		clearDialog : clearContractDetailFields,
		addContractDetail : addContractDetail,
		deleteContractDetail : deleteContractDetail,
		autofillPriceFields : autofillPriceFields,
		saveContract : saveContract,
		initContractDetailFields : initContractDetailFields,
		setLabels : setLabels,
		getLicenseCount : getLicenseCount
	};

})(window);