(function() {
	'use strict';

	var initialized = false;

	var responseType = {
		UNKNOWN_RESP : 0,
		EMPTY_RESP : 1,
		GENERIC_RESP : 2,
		WORKFLOW_RESP : 3
	};
	var iAggregator = 0;
	var editingRow = false;

	$(document).ready(function() {
		if (cvUtil.getTopWindow().document.URL.indexOf("aggregatorManagement.jsp") > 0) {
			iAggregator = 1;
		}
		if (cvUtil.getTopWindow().document.URL.indexOf("partnerManagement.jsp") > 0) {
			iAggregator = 2;
		}
		contractAssociation.init(iAggregator);
		addEvents();
		initGenerateDialog();
		setPermissions();
	});

	function setPermissions() {
		//users with "view" permission should see only selected controls
		var view = true;
		if (typeof (securityPermission) === "undefined" ||
				securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW) {
			view = false;
		}
		$("#assocActionDiv").toggle(view);
		$("#manageContractButton").toggle(view);
		$("#manageAggregatorButton").toggle(view);
		$("#managePartnerButton").toggle(view);

		//users with "view" permission should not be able to access Manage Aggregator & Manage Partner buttons
		//managers should be able to access manage partner & manage service provider pages - MR 273191
		if (typeof (securityPermission) !== "undefined" &&
				(securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW)) {
			$("#manageAggregatorButton").addClass("disabled");
			$("#managePartnerButton").addClass("disabled");
		} else {
			$("#manageAggregatorButton").removeClass("disabled");
			$("#managePartnerButton").removeClass("disabled");
		}
	}

	function openEditAssociationDialog(rowData, iAggregator) {
		contractAssociation.initAssociationDialog(rowData, iAggregator);
		var customerId = rowData[contractUtils.columns.GROUP_ID];
		var height = 280;
		if (iAggregator === 0) {
			$("#divMinFee").hide();
			$("#divBillCycle").hide();
			$("#divUseAsPO").hide();
			$("#vdMasterTable").show();
			height = 580;
		} else if (iAggregator === 1) {
			$("#divBillingGroup").hide();
			$("#vdMasterTable").show();
			height = 620;
			$("#txtMinFee").text(cvUtil.cvLocalize("Minimum_Fee"));
		} else {
			$("#divBillingGroup").hide();
			$("#vdMasterTable").show();
			height = 620;
			$("#txtMinFee").text(cvUtil.cvLocalize("Discount_Percentage"));
		}
		var view = {};
		view.title = '';
		$("#vdDetailTable").empty();
		$('#vdDetailTableWrapper').append(SurveyTemplates.makeReportTable('vdDetailTable', view));
		$("#addVdDetailBtn").val(cvUtil.cvLocalize("contact.add") + " " + cvUtil.cvLocalize("Volume_Discount"));
		$("#clearVdDetailBtn").val(cvUtil.cvLocalize("Clear"));
		clearVolumeDiscount();

		cvUtil.loadPage("getVolumeDiscount.do", {
			reportId : rowData[contractUtils.columns.GROUP_ID],
			reportType : iAggregator
		}, function(data) {
			contractUtils.addActionButtonToTableRows(data);
			contractUtils.addRenderers(data);
			Reports.configureClientSideTable("vdDetailTable", '', data);
			$("#vdDetailTableWrapper .icon-billing").hide();
			$("#vdDetailTableWrapper .icon-send").hide();
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Discount_Details"));

		});

		$("#editAssociationDialog").dialog({
			title : cvUtil.cvLocalize("Title_Edit_Association"),
			height : height,
			width : 850,
			modal : true,
			resizable : false,
			buttons : [
					{
						text : cvUtil.cvLocalize("Close"),
						click : function() {
							$(this).dialog("close");
							$(this).dialog("destroy");
						}
					},
					{
						text : cvUtil.cvLocalize("Save"),
						id : "saveAssociationBtn",
						class : "defaultDialogButton",
						click : function() {
							var discount = {}, tiers = [], group, table, tableData, ctr;
								discount.discountTier = tiers;
						        table = $("#vdDetailTable_table").dataTable();
						        tableData = table.fnGetData();

						        if (!isValidInput(tableData)) {
						            return;
						        }

						        for (ctr = 0; ctr < tableData.length; ctr++) {
						            group = {};
						            group.discountPercentage = tableData[ctr][contractUtils.columns.DISCOUNT_PERCENTAGE] || 0;
						            group.lowerThreshold = tableData[ctr][contractUtils.columns.LOWER_THRESHOLD] || 0;
						            group.upperThreshold = tableData[ctr][contractUtils.columns.UPPER_THRESHOLD] || 0;

						            tiers.push(group);
						        }

							contractAssociation.modifyAssociation($("#editContractList").val(),
									$("#editContractList option:selected").text(),
									customerId,
									$("#editCustomerList").val(),
									$("#editMinFee").val(),
									$("#editBillingTypeList option:selected").val(),
									$("input[name=editUseAsPOType]:checked").val(),
									iAggregator,
									discount);
							$(this).dialog("close");
							$(this).dialog("destroy");
						}
					} ]
		});
	}

	function isValidInput(tableData) {
		var i, j, flag, flag1;
		for (i = 0; i < tableData.length; i++) {
			//check if duplicate discount tiers with same lower threshold or same upper threshold is present
			for (j = i + 1; j < tableData.length; j++) {
				if (parseInt(tableData[i][contractUtils.columns.LOWER_THRESHOLD]) === parseInt(tableData[j][contractUtils.columns.LOWER_THRESHOLD]) ||
						parseInt(tableData[i][contractUtils.columns.UPPER_THRESHOLD]) === parseInt(tableData[j][contractUtils.columns.UPPER_THRESHOLD])) {

					$("#errorMessage").html(cvUtil.cvLocalize("Duplicate_Discount_Tier"));
					$("#errorMessage").addClass("invalidInput");

					return false;
				}
			}
			//check if upper threshold is greater than lower threshold for all rows
			if (parseInt(tableData[i][contractUtils.columns.UPPER_THRESHOLD]) > -1) {
				if (parseInt(tableData[i][contractUtils.columns.LOWER_THRESHOLD]) >= parseInt(tableData[i][contractUtils.columns.UPPER_THRESHOLD])) {

					$("#errorMessage").html(cvUtil.cvLocalize("Invalid_Upper_Threshold"));
					$("#errorMessage").addClass("invalidInput");

					return false;
				}
			}
		}
		if(tableData.length > 0){//check is required only if atleast one discount tier is present. Else consider it to have no volume discount - MR 275285
			//check if rows with LowerThreshold=0 & UpperThreshold=-1 are present
			flag = false;
			for (j = 0; j < tableData.length; j++) {
				if (parseInt(tableData[j][contractUtils.columns.LOWER_THRESHOLD]) === 0) {
					flag = true;
					break;
				}
			}
			flag1 = false;
			for (j = 0; j < tableData.length; j++) {
				if (parseInt(tableData[j][contractUtils.columns.UPPER_THRESHOLD]) === -1) {
					flag1 = true;
					break;
				}
			}
			if (flag === false && flag1 === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Discount_Tier"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			} else if (flag === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Lower_Threshold"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			} else if (flag1 === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Missing_Upper_Threshold"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			}
		}
		//check for continuity of discount tiers
		for (i = 0; i < tableData.length; i++) {
			flag = false;
			for (j = 0; j < tableData.length; j++) {
				if (parseInt(tableData[i][contractUtils.columns.UPPER_THRESHOLD]) <= 0 ||
						parseInt(tableData[i][contractUtils.columns.UPPER_THRESHOLD]) === parseInt(tableData[j][contractUtils.columns.LOWER_THRESHOLD])) {
					flag = true;
					break;
				}
			}
			flag1 = false;
			for (j = 0; j < tableData.length; j++) {
				if (parseInt(tableData[i][contractUtils.columns.LOWER_THRESHOLD]) === 0 ||
						parseInt(tableData[i][contractUtils.columns.LOWER_THRESHOLD]) === parseInt(tableData[j][contractUtils.columns.UPPER_THRESHOLD])) {
					flag1 = true;
					break;
				}
			}
			if (flag === false || flag1 === false) {
				$("#errorMessage").html(cvUtil.cvLocalize("Invalid_Discount_Tier"));
				$("#errorMessage").addClass("invalidInput");

				return false;
			}
		}

		return true;
	}

	function getColumnIndex(table, columnName) {
        var columns = table.fnSettings().aoColumns, i = 0;

        for (i = 0; i < columns.length; i++) {
            if (columns[i].mDataProp === columnName) {
                return i;
            }
        }

        return -1;
    }

	function getFormattedValue(val, o, isUnlimited, isPercentage) {
		if (!val) {
			val = 0;
		}

		if (isUnlimited && (val === -1 || val === 0 || val === "-1")) {
			return cvUtil.cvLocalize("Unlimited");
		} else if (isPercentage){
			return val + "%";
		} else {
			return val;
		}
	}

	Reports.Renderers.discountUnlimitedRenderer = function(o, value) {
		return getFormattedValue(value, o, true, false);
	};

	Reports.Renderers.discountPercentageRenderer = function(o, value) {
		return getFormattedValue(value, o, false, true);
	};

	$("#addVdDetailBtn").click(function() {
		$("#errorMessage").removeClass("invalidInput");
		$("#errorMessage").html("");

		var table = $("#vdDetailTable_table").dataTable(), rows = table.fnGetData(), columnIndex = -1;

		if (editingRow && rows.length > 0){
			var rowIndex = -1;
			rowIndex = contractUtils.global.ROW_INDEX;

			columnIndex = getColumnIndex(table, contractUtils.columns.UPPER_THRESHOLD);
			if ($("#vdUpperLimit").val().trim() === "" || $("#vdUpperLimit").val().trim() === "0") {
				table.fnUpdate("-1", rowIndex, columnIndex);
			} else {
				table.fnUpdate($("#vdUpperLimit").val(), rowIndex, columnIndex);
			}

			columnIndex = getColumnIndex(table, contractUtils.columns.LOWER_THRESHOLD);
			if ($("#vdLowerLimit").val().trim() === "") {
				table.fnUpdate("0", rowIndex, columnIndex);
			} else {
				table.fnUpdate($("#vdLowerLimit").val(), rowIndex, columnIndex);
			}

			columnIndex = getColumnIndex(table, contractUtils.columns.DISCOUNT_PERCENTAGE);
			if ($("#vdDiscount").val().trim() === "") {
				table.fnUpdate("0", rowIndex, columnIndex);
			} else {
				table.fnUpdate($("#vdDiscount").val(), rowIndex, columnIndex);
			}
		} else{
			var row = {};
			if ($("#vdUpperLimit").val().trim() === "" || $("#vdUpperLimit").val().trim() === "0") {
				row[contractUtils.columns.UPPER_THRESHOLD] = "-1";
			} else {
				row[contractUtils.columns.UPPER_THRESHOLD] = $("#vdUpperLimit").val();
			}
			if ($("#vdLowerLimit").val().trim() === "") {
				row[contractUtils.columns.LOWER_THRESHOLD] = "0";
			} else {
				row[contractUtils.columns.LOWER_THRESHOLD] = $("#vdLowerLimit").val();
			}
			if ($("#vdDiscount").val().trim() === "") {
				row[contractUtils.columns.DISCOUNT_PERCENTAGE] = "0";
			} else {
				row[contractUtils.columns.DISCOUNT_PERCENTAGE] = $("#vdDiscount").val();
			}
			row["ACTION_COL"] = "";

			table.fnAddData(row);
			table.fnSettings()._iDisplayLength = table.fnGetData().length;
			table.fnDraw();
		}
		$("#vdDetailTableWrapper .icon-billing").hide();
		$("#vdDetailTableWrapper .icon-send").hide();
	});

	$("#clearVdDetailBtn").click(function() {
		if ($(this).val().trim() === cvUtil.cvLocalize("Add_New")) {
			$("#addVdDetailBtn").val(cvUtil.cvLocalize("contact.add") + " " + cvUtil.cvLocalize("Volume_Discount"));
			$("#clearVdDetailBtn").val(cvUtil.cvLocalize("Clear"));
		}

		clearVolumeDiscount();
	});

	function clearVolumeDiscount() {
		$("#vdUpperLimit").val("");
		$("#vdLowerLimit").val("");
		$("#vdDiscount").val("");

		editingRow=false;

		$("#errorMessage").removeClass("invalidInput");
		$("#errorMessage").html("");
	}

	$("#vdDetailTableWrapper").on("click",
			'#editItemButton',
			function() {
				var rowIndex = $("#vdDetailTable_table").dataTable().fnGetPosition($(this).closest('tr')
						.find('td')[0])[0];
				var rowData = $("#vdDetailTable_table").dataTable().fnGetData()[rowIndex];

				contractUtils.global.ROW_INDEX = rowIndex;//placeholder for rowIndex to edit pricing

				$("#addVdDetailBtn").val(cvUtil.cvLocalize("Update_Discount"));
				$("#clearVdDetailBtn").val(cvUtil.cvLocalize("Add_New"));
				$("#vdDiscount").val(rowData[contractUtils.columns.DISCOUNT_PERCENTAGE]);
				$("#vdUpperLimit").val(rowData[contractUtils.columns.UPPER_THRESHOLD]);
				$("#vdLowerLimit").val(rowData[contractUtils.columns.LOWER_THRESHOLD]);

				editingRow = true;
			});

	$("#vdDetailTableWrapper").on("click", '#deleteItemButton', function() {
		var row = $(this).closest('tr');
		var table = $("#vdDetailTable_table").dataTable();
		var index = table.fnGetPosition(row.find('td')[0])[0];
		table.fnDeleteRow(index);
	});

	function openDeleteAssociationDialog(rowData, iAggregator) {
		var columnName = "";
		if (iAggregator === 0) {
			columnName = contractUtils.columns.CUSTOMER_NAME;
		} else if (iAggregator === 1) {
			columnName = contractUtils.columns.AGGREGATOR_NAME;
		} else {
			columnName = contractUtils.columns.PARTNER_NAME;
		}
		$("#deleteAssocMessage").html(cvUtil.cvLocalize("Confirm_Remove_Association", rowData[columnName]));

		$("#delete-association-dialog-form").dialog({
			title : cvUtil.cvLocalize("Title_Confirm_Delete"),
			height : 150,
			width : 500,
			modal : true,
			resizable : false,
			buttons : [ {
				text : cvUtil.cvLocalize("No"),
				id : "doNotDeleteAssociation",
				click : function() {
					$(this).dialog("close");
					$(this).dialog("destroy");
				}
			}, {
				text : cvUtil.cvLocalize("Yes"),
				id : "deleteAssociation",
				click : function() {
					contractAssociation.deleteAssociation(rowData[contractUtils.columns.GROUP_ID], iAggregator);
					$(this).dialog("close");
					$(this).dialog("destroy");
				}
			} ]
		});
	}

	function openBillingDialog(data, email, iAggregator) {
		var type = 0;
		type = iAggregator;

		var title = "", btnText = "";
		var width = 0, height = 0;

		if (email === false) {
			title = cvUtil.cvLocalize("Generate") + " " + cvUtil.cvLocalize("Royalty_Report");
			btnText = cvUtil.cvLocalize("Ok");
			width = 460;
			if(type === 2){
				height = 280;
				$("#hideSummaryBox").show();
			}else{
				height = 260;
				$("#hideSummaryBox").hide();
			}
		} else {
			title = cvUtil.cvLocalize("Send") + " " + cvUtil.cvLocalize("Royalty_Report");
			btnText = cvUtil.cvLocalize("Send");
			width = 670;

			if (iAggregator === 0 || iAggregator === 2) {
				height = 360;

				$("#notifyLabel").hide();
				$("#notifyEmail").hide();

				if (iAggregator === 2) {
					$("#custNameLabel").text(cvUtil.cvLocalize("Customer_Name", cvUtil.cvLocalize("Partner")));
					$("#custEmailLabel").text(cvUtil.cvLocalize("Customer_Email", cvUtil.cvLocalize("Partner")));
				} else {
					$("#custNameLabel").text(cvUtil.cvLocalize("Customer_Name", cvUtil.cvLocalize("Billing_Group")));
					$("#custEmailLabel").text(cvUtil.cvLocalize("Customer_Email", cvUtil.cvLocalize("Billing_Group")));
				}
			} else if (iAggregator === 1) {
				height = 390;

				$("#notifyLabel").show();
				$("#notifyEmail").show();

				$("#custNameLabel").text(cvUtil.cvLocalize("Customer_Name", cvUtil.cvLocalize("Service_Provider")));
				$("#custEmailLabel").text(cvUtil.cvLocalize("Customer_Email", cvUtil.cvLocalize("Service_Provider")));
			}
		}
		$("#generateBillDialog")
				.dialog({
					title : title,
					height : height,
					width : width,
					modal : true,
					resizable : false,
					open : function() {
						if (email === true) {
							$("#generateOptions").hide();
							$("#emailInputs").show();
							$("#detailsBox").hide();
						} else {
							$("#generateOptions").show();
							$("#emailInputs").hide();
							$("#detailsBox").show();
						}
					},
					buttons : [
							{
								text : cvUtil.cvLocalize("Cancel"),
								click : function() {
									$(this).dialog("close");
									$(this).dialog("destroy");
								}
							},
							{
								text : btnText,
								class : "defaultDialogButton",
								click : function() {
									var optType = $("input[name=optType]:checked", "#generateOptions").val(), customerId = data[contractUtils.columns.GROUP_ID], month = $("#monthSelect")
											.val(), year = parseInt($("#yearSelect option:selected").text()), quarter = 0, custName = "", custEmail = "", cvltName = "", cvltEmail = "", notifyEmail = "", useAsPO = "0", includeDetails = 0;

									if (iAggregator !== 0) {
										if (data[contractUtils.columns.BILLING_TYPE].toLowerCase().trim() !== cvUtil
												.cvLocalize("Monthly").toLowerCase().trim()) {
											quarter = 1;
										}
									}

									includeDetails = 0;//starting sp15, details will be shown using custom reports. Hence old logic is deprecated

									//MR 249464
									var hideSummary=0
									if (type === 2) {
										if ($('input[name="hideSummary"]').is(':checked')) {
											hideSummary = 1;
										}
									}

									if (email === true) {
										custName = $("#custName").val().trim().toLowerCase();
										custEmail = $("#custEmail").val().trim().toLowerCase();
										cvltName = $("#cvltName").val().trim().toLowerCase();
										cvltEmail = $("#cvltEmail").val().trim().toLowerCase();
										notifyEmail = $("#notifyEmail").val().trim().toLowerCase();

										var proceed = true;
										if (custName.trim() === "" || custEmail.trim() === "" ||
												cvltName.trim() === "" || cvltEmail.trim() === "") {
											cvUtil.errorToast(cvUtil.cvLocalize("Incorrect_Details"));
											proceed = false;
										} else {
											if (!cvUtil.isEmailValid(custEmail) || !cvUtil.isEmailValid(cvltEmail)) {
												cvUtil.errorToast(cvUtil.cvLocalize("Invalid_Email"));
												proceed = false;
											}
										}
										//check if notify field has valid entries
										if (notifyEmail.trim() !== "") {
											var validNotify = true;
											notifyEmail = notifyEmail.replace(/;/g, ',');//if email ids are separated by semicolon, make it a comma separated list
											notifyEmail = notifyEmail.replace(/ /g, '');//remove white spaces in between the comma separated list
											//remove comma from the beginning and end
											//if (notifyEmail.substring(0, 1) === ',') {
											if (notifyEmail.slice(-1) === ',') {
												notifyEmail = notifyEmail.slice(0, notifyEmail.length - 1);
											}
											//if (notifyEmail.substring(notifyEmail.length - 1, 1) === ',') {
											if (notifyEmail.slice(0, 1) === ',') {
												notifyEmail = notifyEmail.slice(1, notifyEmail.length);
											}

											//now check if each entry in the list is a valid email
											var emailArray = notifyEmail.split(',');
											$.each(emailArray, function(i, val) {
												if (!cvUtil.isEmailValid(val)) {
													validNotify = false;
												}
											});
											if (validNotify === false) {
												cvUtil.errorToast(cvUtil.cvLocalize("Invalid_Email"));
												proceed = false;
											}
										}

										if (proceed === true) {
											$(this).dialog("close");
											$(this).dialog("destroy");

											$("#content").mask(cvUtil.cvLocalize("Loading"));
											cvUtil
													.loadPageWithSessionCheck("sendReport.do",
															{
																customerName : custName,
																customerEmail : custEmail,
																commvaultName : cvltName,
																commvaultEmail : cvltEmail,
																customerId : customerId,
																month : month,
																year : year,
																quarter : quarter,
																useAsPO : useAsPO,
																type : type,
																notifyEmail : notifyEmail,
																includeDetails : includeDetails
															},
															function(data) {
																if (data) {
																	if (typeof data.errorMessage === "undefined" ||
																			data.errorMessage.trim() === "") {
																		data.errorMessage = cvUtil
																				.cvLocalize("error.msg.Unknown_error_check_log_files");
																	}
																	if (data.respType === responseType.WORKFLOW_RESP) {
																		if (data.status === 4) {
																			cvUtil.toast(cvUtil
																					.cvLocalize("Send_Report_Success"));
																		} else {
																			cvUtil.errorToast(data.errorMessage);
																			cvUtil.errorToast(cvUtil
																					.cvLocalize("Send_Report_Failure"));
																		}
																	} else {
																		if (typeof data.errorCode === "undefined") {
																			data.errorCode = -2;
																		}
																		if (data.errorCode === 0) {
																			cvUtil.toast(cvUtil
																					.cvLocalize("Send_Report_Success"));
																		} else {
																			cvUtil.errorToast(data.errorMessage);
																			cvUtil.errorToast(cvUtil
																					.cvLocalize("Send_Report_Failure"));
																		}
																	}
																} else {
																	cvUtil.errorToast(cvUtil
																			.cvLocalize("Send_Report_Failure"));
																}
																$("#content").unmask();
															},
															function(xhr, ajaxOptions, thrownError) {
																cvUtil.errorToast(cvUtil
																		.cvLocalize("Send_Report_Failure"));
																$("#content").unmask();

															});
										}
									} else {
										if (optType === "pdf") {
											var formName = "#exportBillToPDFForm";
											var form = $(formName);
											$(formName + " #isExportBill").val(true);
											$(formName + " #customerId").val(customerId);
											$(formName + " #month").val(month);
											$(formName + " #year").val(year);
											$(formName + " #quarter").val(quarter);
											$(formName + " #type").val(type);
											if (type === 2) {
												$(formName + " #hideSummary").val(hideSummary);
											}

											form.attr('action', cvUtil.getContextPath() + '/cloud/contractmanagement/exportBillToPDF.do');
											form.submit();

										} else if (optType === "html"){
											var billingUrl = cvUtil.getContextPath() +
											"/billing/billing.jsp?customerId=" + customerId + "&month=" +
											month + "&year=" + year + "&quarter=" + quarter + "&type=" + type;
											if (type === 2) {
												billingUrl+="&hideSummary=" + hideSummary;
											}
											cvUtil.getTopWindow().open(billingUrl);

										} else if (optType === "rpt"){
											var monthString=month;
											if(parseInt(month) < 10){
												monthString = "0" + monthString;
											}
											var rptUrl = "/reportsplus/reportViewer.jsp?reportId=Billing%20Details&input.type=" + type + "&input.reportId=" + customerId + "&input.billCycle=" + quarter + "&input.month=" + year + "-" + monthString + "-01";
											cvUtil.getTopWindow().open(cvUtil.getContextPath() + rptUrl);
										}
										$(this).dialog("close");
										$(this).dialog("destroy");
									}
								}
							} ]
				});
	}

	function initGenerateDialog() {
		var today = new Date(), curYear = today.getFullYear(), curMonth = today.getMonth(), totalYear = 10, yearSelect = $("#yearSelect"), i = 0, optEle;

		for (i = 0; i < totalYear; i++, curYear--) {
			yearSelect.append("<option value=" + curYear + ">" + curYear + "</option>");
		}
		//disable future months
		$("#monthSelect option").each(function(i) {
			if ($(this).val() > curMonth + 1) {
				$("#monthSelect option[value='" + $(this).val() + "']").attr("disabled", true);
			}
		});
		//select current month as default
		$("#monthSelect option[value='" + (curMonth + 1) + "']").attr("selected", "selected");
	}

	function addEvents() {

		$('#yearSelect').on('change', function() {
			var today = new Date(), curYear = today.getFullYear(), curMonth = today.getMonth();
			if (parseInt(this.value) < curYear) {//enable all months for past years
				$("#monthSelect option").each(function(i) {
					$("#monthSelect option[value='" + (i + 1) + "']").attr("disabled", false);
				});
			} else if (parseInt(this.value) === curYear) {//disable future months for present year
				$("#monthSelect option").each(function() {
					if ($(this).val() > curMonth + 1) {
						$("#monthSelect option[value='" + $(this).val() + "']").attr("disabled", true);
					}
				});
				//for current year - if the already selected month is greater than current month, make current month as default selection
				if ($("#monthSelect").find(":selected").val() > curMonth + 1) {
					$("#monthSelect option[value='" + (curMonth + 1) + "']").attr("selected", "selected");
				}
			}
		});

		$("#customerTableWrapper").on("click", '#editItemButton', function() {
			var rowIndex = $("#customerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#customerTable_table").dataTable().fnGetData()[rowIndex];
			openEditAssociationDialog(rowData, 0);
		});
		$("#aggregatorTableWrapper").on("click",
				'#editItemButton',
				function() {
					var rowIndex = $("#aggregatorTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var rowData = $("#aggregatorTable_table").dataTable().fnGetData()[rowIndex];
					openEditAssociationDialog(rowData, 1);
				});
		$("#partnerTableWrapper").on("click", '#editItemButton', function() {
			var rowIndex = $("#partnerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#partnerTable_table").dataTable().fnGetData()[rowIndex];
			openEditAssociationDialog(rowData, 2);
		});
		$("#customerTableWrapper").on("click", '#deleteItemButton', function() {
			var rowIndex = $("#customerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#customerTable_table").dataTable().fnGetData()[rowIndex];
			openDeleteAssociationDialog(rowData, 0);
		});
		$("#aggregatorTableWrapper").on("click",
				'#deleteItemButton',
				function() {
					var rowIndex = $("#aggregatorTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var rowData = $("#aggregatorTable_table").dataTable().fnGetData()[rowIndex];
					openDeleteAssociationDialog(rowData, 1);
				});
		$("#partnerTableWrapper").on("click", '#deleteItemButton', function() {
			var rowIndex = $("#partnerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#partnerTable_table").dataTable().fnGetData()[rowIndex];
			openDeleteAssociationDialog(rowData, 2);
		});

		$("#customerTableWrapper").on("click", '#generateBillButton,#emailBillButton', function() {
			var rowIndex = $("#customerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#customerTable_table").dataTable().fnGetData()[rowIndex];
			var email = false;
			if (this.id === "emailBillButton") {
				email = true;
			}
			openBillingDialog(rowData, email, 0);
		});

		$("#aggregatorTableWrapper").on("click",
				'#generateBillButton,#emailBillButton',
				function() {
					var rowIndex = $("#aggregatorTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var rowData = $("#aggregatorTable_table").dataTable().fnGetData()[rowIndex];
					var email = false;
					if (this.id === "emailBillButton") {
						email = true;
					}
					openBillingDialog(rowData, email, 1);
				});
		$("#partnerTableWrapper").on("click", '#generateBillButton,#emailBillButton', function() {
			var rowIndex = $("#partnerTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#partnerTable_table").dataTable().fnGetData()[rowIndex];
			var email = false;
			if (this.id === "emailBillButton") {
				email = true;
			}
			openBillingDialog(rowData, email, 2);
		});
		$(".integerInput").keypress(function(e) {
			if (e.which !== 8 && e.which !== 0 && (e.which < 48 || e.which > 57)) {
				return false;
			}
		});

		$(".decimalInput").keypress(function(e) {
			if (e.which === 46) {
				return true;
			}

			if (e.which !== 8 && e.which !== 0 && (e.which < 48 || e.which > 57)) {
				return false;
			}
		});

		$("#associateButton").click(function() {
			contractAssociation.addAssociation($("#contractList").val(),
					$("#contractList option:selected").text(),
					$("#customerList").val(),
					$("#customerList option:selected").text(),
					$("#minFee").val(),
					$("#billingTypeList option:selected").val(),
					$("input[name=useAsPOType]:checked").val(),
					0);

		});

		$("#associateAggregatorButton").click(function() {
			contractAssociation.addAssociation($("#contractList").val(),
					$("#contractList option:selected").text(),
					$("#aggregatorList").val(),
					$("#aggregatorList option:selected").text(),
					$("#minFee").val(),
					$("#billingTypeList option:selected").val(),
					$("input[name=useAsPOType]:checked").val(),
					1);

		});

		$("#associatePartnerButton").click(function() {
			contractAssociation.addAssociation($("#contractList").val(),
					$("#contractList option:selected").text(),
					$("#partnerList").val(),
					$("#partnerList option:selected").text(),
					$("#minFee").val(),
					$("#billingTypeList option:selected").val(),
					$("input[name=useAsPOType]:checked").val(),
					2);

		});
		$("#manageContractButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/manageContract.jsp");
		});
		$("#aggregatorRedirectButton")
				.click(function() {
					cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() +
							"/cloud/contract/aggregatorManagement.jsp");
				});
		$("#ccGroupRedirectButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/contractManagement.jsp");
		});
		$("#manageAggregatorButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/manageAggregator.jsp");
		});
		$("#partnerRedirectButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/partnerManagement.jsp");
		});
		$("#managePartnerButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/managePartner.jsp");
		});

		function updateActionButtons(actionButton, isEnabled) {
			if (isEnabled) {
				actionButton.removeClass("disabled");
			} else {
				actionButton.addClass("disabled");
			}
		}
	}

}(this));