(function() {
	'use strict';

	var initialized = false;

	$(document).ready(function() {
		createContractTable(true);
		addEvents();
	});
	function createContractTable(startup) {
		var view = {};
		view.title = '';
		view.filters = true;
		view.collapsible = false;
		view.alarm = false;
		view.exportExcel = true;
		view.colChooser = true;
		$("#contractTable").empty();
		$('#contractTableWrapper').append(SurveyTemplates.makeReportTable('contractTable', view));
		getContracts(startup);
	}

	function getContracts(startup) {
		$("#content").mask(cvUtil.cvLocalize("Loading"));
		cvUtil.loadPage("getContracts.do", {
			option : 1
		}, function(data) {
			contractUtils.addActionButtonToTableRows(data);
			contractUtils.addRenderers(data);
			createTable(data, 'contractTable');
			setPermissions();
			$("#content").unmask();

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Contracts"));
			$("#content").unmask();
		}, {
		//"async" : false
		});
	}

	function createTable(data, tableDiv, title) {
		contractUtils.disbleSorting(data.aoColumns);
		//Reports.configureClientSideTable(tableDiv, '', data);
		$('#contractTableWrapper #' + tableDiv + '_table_length').remove();
		data.iDisplayLength = 100;
		Reports.configureClientSideTableWithPaging(tableDiv, '', data);
		$("#" + tableDiv + " .rptTblBtnEncDiv").append($("#" + tableDiv + "_table_length"));
		$("#contractTableWrapper .icon-billing").hide();
		$("#contractTableWrapper .icon-send").hide();
	}

	function setPermissions() {
		//users with "view" permission should see only selected controls
		var view = true;
		if (typeof (securityPermission) === "undefined" ||
				securityPermission.toLowerCase().trim() === contractUtils.permissions.VIEW) {
			view = false;
		}
		$("#tableHeader").toggle(view);
		$("#contractTableWrapper").toggle(view);
		$("#noPermission").toggle(!view);

		//users with "manage" or "view" permission should not be able to access approve checkbox & Manage Billing Group button
		if (typeof (securityPermission) !== "undefined" &&
				securityPermission.toLowerCase().trim() === contractUtils.permissions.ADMIN) {
			$("#contractTable .chkApprove").removeClass("disabled");
			$("#billingGroupButton").removeClass("disabled");
		} else {
			$("#contractTable .chkApprove").addClass("disabled");
			$("#billingGroupButton").addClass("disabled");
		}
	}

	function openDeleteContractEntityDialog(rowData) {
		$("#entityContractName").html(rowData[contractUtils.columns.CONTRACT]);
		$("#groupName").html(rowData[contractUtils.columns.BILLING_GROUP]);
		$("#countingType").html(rowData[contractUtils.columns.COUNTING_TYPE]);
		$("#entityType").html(rowData[contractUtils.columns.ENTITY_TYPE]);
		$("#entityPrice").html(rowData[contractUtils.columns.UNIT_PRICE]);
		$("#entityBaseline").html(rowData[contractUtils.columns.BASELINE]);
		$("#entityLowerLimit").html(rowData[contractUtils.columns.LOWER_LIMIT]);
		$("#entityUpperLimit").html(rowData[contractUtils.columns.UPPER_LIMIT]);

		$("#delete-contract-entity-dialog-form").dialog({
			title : cvUtil.cvLocalize("Title_Confirm_Delete"),
			height : 375,
			width : 500,
			modal : true,
			resizable : false,
			buttons : [ {
				text : cvUtil.cvLocalize("Yes"),
				id : "deleteContractButton",
				click : function() {
					deleteContract(rowData);
				}
			}, {
				text : cvUtil.cvLocalize("No"),
				id : "doNotDeleteContractButton",
				click : function() {
					$(this).dialog("close");
					$(this).dialog("destroy");
				}
			} ]
		});
	}

	function showConfirmDeleteOnSave() {
		$("#save-and-delete-dialog-form").dialog({
			title : cvUtil.cvLocalize("Confirm"),
			height : 1500,
			width : 350,
			modal : true,
			resizable : false,
			buttons : [
					{
						text : cvUtil.cvLocalize("Yes"),
						id : "saveAndDeleteButton",
						click : function() {
							contractDetailDialog.saveContract(2, true, [
									closeConfirmOnSaveDialog,
									closeDetailDialog,
									getContracts ]);
						}
					},
					{
						text : cvUtil.cvLocalize("No"),
						id : "doNotDeleteButton",
						click : function() {
							$(this).dialog("close");
							$(this).dialog("destroy");
						}
					} ]
		});
	}

	function openContractDetailDialog(isEdit, rowData) {
		var title = cvUtil.cvLocalize("Title_New_Contract");
		if (isEdit) {
			title = cvUtil.cvLocalize("Title_Edit_Contract") + " - " + rowData[contractUtils.columns.CONTRACT];
		}
		contractDetailDialog.initDialog(isEdit, rowData);
		$("#contract-dialog-form").dialog({
			title : title,
			height : 750,
			width : 850,
			modal : true,
			resizable : false,
			buttons : [ {
				text : cvUtil.cvLocalize("Close"),
				click : function() {
					closeDetailDialog();
				}
			}, {
				text : cvUtil.cvLocalize("Save"),
				id : "newContractOkBtn",
				class : "defaultDialogButton",
				click : function() {
					contractDetailDialog.saveContract(isEdit ? 2 : 1, false, [ closeDetailDialog, getContracts ]);
				}
			} ]
		});
	}

	function closeConfirmOnSaveDialog() {
		$("#save-and-delete-dialog-form").dialog("close");
		$("#save-and-delete-dialog-form").dialog("destroy");
	}

	function closeDetailDialog() {
		$("#contract-dialog-form").dialog("close");
		$("#contract-dialog-form").dialog("destroy");
	}

	function deleteContract(contract) {
		cvUtil.loadPageWithSessionCheck("deleteContract.do", {
			contractId : contract[contractUtils.columns.CONTRACT_ID]
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(cvUtil.cvLocalize("Delete_Contract_Success"));
				getContracts();
			} else {
				cvUtil.errorToast(cvUtil.cvLocalize("Delete_Contract_Failure"));
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Delete_Contract_Failure"));

		});
	}

	function addEvents() {
		$("#addContractButton").click(function(event) {
			openContractDetailDialog(false, null);

		});

		$("#addContractDetailBtn").click(function() {
			contractDetailDialog.addContractDetail();
		});

		$("#clearContractDetailBtn").click(function() {
			var isNew = false;
			if ($(this).val().trim() === cvUtil.cvLocalize("contact.Add") + " " + cvUtil.cvLocalize("New")) {
				isNew = true;
			}
			contractDetailDialog.clearDialog(isNew);
		});

		$("#contractTableWrapper").on("click", '#editItemButton', function() {
			var rowIndex = $("#contractTable_table").dataTable().fnGetPosition($(this).closest('tr').find('td')[0])[0];
			var rowData = $("#contractTable_table").dataTable().fnGetData()[rowIndex];
			openContractDetailDialog(true, rowData);
		});

		$("#contractTableWrapper").on("click",
				'#deleteItemButton',
				function() {
					var rowIndex = $("#contractTable_table").dataTable()
							.fnGetPosition($(this).closest('tr').find('td')[0])[0];
					var rowData = $("#contractTable_table").dataTable().fnGetData()[rowIndex];
					contractUtils.openDeleteConfirmDialog(rowData,
							cvUtil.cvLocalize("Confirm_Delete_Contract_Association"),
							rowData[contractUtils.columns.CONTRACT],
							deleteContract);
				});

		$("#contractTableWrapper").on("click", '.tableExportExcel', function() {
			getContracts();//if approval status is changed after initial page load, it will not be reflected in the excel. Hence contracts table should be reloaded before export
			var dataTable = $("#contractTable_table").dataTable();
			contractUtils.exportToExcel(dataTable, contractUtils.global.CONTRACTS);
		});

		$("#contractDetailTableWrapper").on("click",
				'#editItemButton',
				function() {
					var rowIndex = $("#contractDetailTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var dialogData = $("#contractDetailTable_table").dataTable().fnGetData()[rowIndex];

					contractUtils.global.ROW_INDEX = rowIndex;//placeholder for rowIndex to edit pricing
					contractDetailDialog.initContractDetailFields(dialogData);
				});

		$("#contractDetailTableWrapper").on("click", '#deleteItemButton', function() {
			var row = $(this).closest('tr');
			contractDetailDialog.deleteContractDetail(row);
		});

		$("#billingGroupList")
				.change(function() {
					var countingType = $("#billingGroupList option:selected").attr("data-counting-type");
					$("#contract-dialog-form #countingType").val(countingType);
					$("#contract-dialog-form #entityType").val($("#billingGroupList option:selected")
							.attr("data-entity-type"));
					contractDetailDialog.setLabels(countingType === cvUtil.cvLocalize("Capacity"));
					contractDetailDialog.autofillPriceFields();
				});

		$("#contractName").keypress(function() {
			$("#contractName").removeClass("errorInput");
			$("#errorMessage").html("");
		});

		$(document).on("keypress", '.integerInput', function(e) {
			if (e.which !== 8 && e.which !== 0 && (e.which < 48 || e.which > 57)) {
				return false;
			}
		});

		$(document).on("keypress", '.decimalInput', function(e) {
			if (e.which === 46) {
				return true;
			}

			if (e.which !== 8 && e.which !== 0 && (e.which < 48 || e.which > 57)) {
				return false;
			}
		});

		$("#contractTable tbody tr").click(function() {
			$("#contractTable tbody tr").removeClass('rowSelected');
			$(this).addClass('rowSelected');
		});

		$("#contractTable tbody").click(function() {
			$("#contractTable tbody tr").removeClass('rowSelected');
			$(this).addClass('rowSelected');
		});

		$("#contractTable .tableFilter").click(function(event) {
			if (!initialized) {
				var gridId = $(this).attr('id');
				gridId = gridId.substring(0, gridId.lastIndexOf("_"));
				Reports.tableFilterColumns(gridId);
				$(this).toggleClass("filterOn");
				initialized = true;
			}
			$('input#contractTable_filterText-10').remove();
			$('input#contractTable_filterText-11').remove();

		});

		$("#billingGroupButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/manageBillingGroup.jsp");
		});
		$("#aggregatorRedirectButton")
				.click(function() {
					cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() +
							"/cloud/contract/aggregatorManagement.jsp");
				});
		$("#ccGroupRedirectButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/contractManagement.jsp");
		});
		function updateActionButtons(actionButton, isEnabled) {
			if (isEnabled) {
				actionButton.removeClass("disabled");
			} else {
				actionButton.addClass("disabled");
			}
		}

		$("#contractTable").on("click", ".selectCB", function() {
			var contractId = $(this).attr("data-contractid");
			var status = $(this).hasClass("unchecked") ? 1 : 0;
			updateActionButtons($(this).parent().siblings().find(".rowActionButton"), !status);

			cvUtil.loadPageWithSessionCheck("setApproveStatus.do", {
				contractId : contractId,
				status : status
			}, function(data) {
				if (data && data.errorCode === 0) {
					cvUtil.toast(cvUtil.cvLocalize("Approval_Status_Update_Success"));
				} else {
					cvUtil.errorToast(cvUtil.cvLocalize("Approval_Status_Update_Failure"));
				}
			}, function(xhr, ajaxOptions, thrownError) {
				cvUtil.errorToast(cvUtil.cvLocalize("Approval_Status_Update_Failure"));
			});
		});

	}

	Reports.Renderers.contractCapacityRenderer = function(o, val) {
		if (!val) {
			val = 0;
		}

		if (o.aData[contractUtils.columns.COUNTING_TYPE] === cvUtil.cvLocalize("Capacity")) {
			return val + " TB";
		} else {
			return val;
		}
	};

	function getFormattedValue(val, o, isUnlimited, showUnit) {
		var unit = "";

		if (showUnit && o.aData[contractUtils.columns.COUNTING_TYPE] === cvUtil.cvLocalize("Capacity")) {
			unit = " TB";
		}

		if (!val) {
			val = 0;
		}

		if (isUnlimited && (val === -1 || val === 0 || val === "-1")) {
			return cvUtil.cvLocalize("Unlimited");
		} else {
			return val + unit;
		}
	}

	function getMultiLineValue(o, val, isUnlimited, showUnit) {
		var values = val.split(","), newVal = "";

		if (values.length === 1) {
			return getFormattedValue(val, o, isUnlimited);
		}

		$.each(values, function(i, v) {
			if (i !== 0) {
				newVal = newVal + "<br>";
			}

			newVal = newVal + getFormattedValue(v, o, isUnlimited, showUnit);

		});

		return newVal;
	}

	Reports.Renderers.contractMultiLineRenderer = function(o, value) {
		return getMultiLineValue(o, value, false, false);
	};

	Reports.Renderers.contractMultiLineRendererWithUnit = function(o, value) {
		return getMultiLineValue(o, value, false, true);
	};

	Reports.Renderers.contractMultiLineRendererUnlimited = function(o, value) {
		return getMultiLineValue(o, value, true, true);
	};

	Reports.Renderers.contractPriceUnlimitedRenderer = function(o, value) {
		return getFormattedValue(value, o, true, true);
	};

}(this));