(function() {
	'use strict';
	var entities = {};

	$(document).ready(function() {
		createAggregatorTable(true);
		addEvents();
	});
	function createAggregatorTable(startup) {
		var view = {};
		view.title = '';
		view.filters = false;
		view.collapsible = false;
		view.alarm = false;
		view.exportExcel = true;
		view.colChooser = true;
		$("#aggregatorTable").empty();
		$('#aggregatorTableWrapper').append(SurveyTemplates.makeReportTable('aggregatorTable', view));
		getAggregators(startup);
	}

	function getAggregators() {
		$("#content").mask(cvUtil.cvLocalize("Loading"));
		cvUtil.loadPage("getAggregators.do", {}, function(data) {
			contractUtils.addActionButtonToTableRows(data);
			contractUtils.addRenderers(data);
			createTable(data, 'aggregatorTable');
			setPermissions();
			$("#content").unmask();
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Aggregator"));
			$("#content").unmask();

		}, {
			"async" : false
		});
	}

	function createTable(data, tableDiv) {
		contractUtils.disbleSorting(data.aoColumns);
		Reports.configureClientSideTable(tableDiv, '', data);
		$("#aggregatorTableWrapper .icon-billing").hide();
		$("#aggregatorTableWrapper .icon-send").hide();
	}

	function setPermissions() {
		//users with "view" permission should see only selected controls
		//managers should be able to access manage partner page - MR 273191
		var view = false;
		if (typeof (securityPermission) !== "undefined" &&
				(securityPermission.toLowerCase().trim() === contractUtils.permissions.ADMIN || securityPermission.toLowerCase().trim() === contractUtils.permissions.MANAGE)) {
			view = true;
		}
		$("#tableHeader").toggle(view);
		$("#aggregatorTableWrapper").toggle(view);
		$("#noPermission").toggle(!view);

	}

	function openGroupDetailDialog(rowData) {
		var title = "";
		if (rowData) {
			title = cvUtil.cvLocalize("contact.edit") + " " + cvUtil.cvLocalize("Service_Provider");
		} else {
			title = cvUtil.cvLocalize("New") + " " + cvUtil.cvLocalize("Service_Provider");
		}
		initDialog(rowData);
		$("#aggregator-dialog-form").dialog({
			title : title,
			height : 510,
			width : 700,
			modal : true,
			resizable : false,
			buttons : [ {
				text : cvUtil.cvLocalize("Close"),
				click : function() {
					$(this).dialog("close");
					$(this).dialog("destroy");
				}
			}, {
				text : cvUtil.cvLocalize("Save"),
				class : "defaultDialogButton",
				click : function() {
					if (isValidInput()) {
						saveAggregator(rowData);
					}
				}
			} ]
		});
	}

	function initDialog(rowData) {
		$("#messageLabel").text("");
		if (rowData) {
			$("#aggregator-dialog-form").mask(cvUtil.cvLocalize("Loading"));
			cvUtil.loadPage("getAggregatorDetail.do", {
				groupId : rowData[contractUtils.columns.AGGREGATOR_ID]
			}, function(data) {
				$("#aggregator-dialog-form").unmask();
				if (data && data.aaData && data.aaData.length > 0) {
					var groupData = data.aaData[0];
					$("#aggregatorName").val(groupData[contractUtils.columns.AGGREGATOR_NAME]);
					$("#aggregatorDesc").val(groupData[contractUtils.columns.DESCRIPTION]);
					$.when(populateCCGrpsList()).then(function() {
						initCCGrpsDetailPanel(data.aaData);
					});

				} else {
					cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Aggregator_Details"));
				}
			}, function(xhr, ajaxOptions, thrownError) {
				$("#aggregator-dialog-form").unmask();
				cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Aggregator_Details"));
			});

		} else {
			$("#aggregatorName").val("");
			$("#aggregatorDesc").val("");
			populateCCGrpsList();
		}
	}

	function sortList($list) {
		$list.children().detach().sort(function(a, b) {
			return $(a).text().localeCompare($(b).text());
		}).appendTo($list);
	}

	function isValidInput() {
		if ($("#aggregatorName").val().trim() === "") {
			$("#messageLabel").text(cvUtil.cvLocalize("Enter_Aggregator_Name"));
			$("#aggregatorName").focus();
			return false;
		}

		if ($(".selectedList li").length === 0) {
			$("#messageLabel").text(cvUtil.cvLocalize("Add_CCGroups"));
			return false;
		}

		return true;

	}

	function saveAggregator(rowData) {
		var group = {}, option = 1, isEdit = false;
		if (rowData) {
			isEdit = true;
			option = 2;
		}

		//we are reusing .x structure of BillingGroup for Aggregators
		//Hence the below property names should be that of "Message BillingGroup" in WebReport.x
		group.billingGroupName = $("#aggregatorName").val();
		group.description = $("#aggregatorDesc").val() || "";
		group.countingType = "0";
		group.licenseList = [];
		$(".selectedList li").each(function() {
			var license = {};
			license.licenseId = $(this).attr("data-id");
			license.licenseName = $(this).attr("data-name");
			license.uniqueId = 0;
			license.licenseType = 0;
			group.licenseList.push(license);
		});

		if (isEdit) {
			group.billingGroupId = rowData[contractUtils.columns.AGGREGATOR_ID];
		}

		cvUtil.loadPageWithPOST("saveAggregator.do", {
			aggregator : JSON.stringify(group),
			option : option
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(data.message);
				$("#aggregator-dialog-form").dialog("close");
				$("#aggregator-dialog-form").dialog("destroy");
				getAggregators();
			} else {
				cvUtil.errorToast(data.message);
			}
		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Save_Aggregator_Failure"));
		});
	}

	var deleteAggregator = function(rowData) {
		cvUtil.loadPage("deleteAggregator.do", {
			groupId : rowData[contractUtils.columns.AGGREGATOR_ID]
		}, function(data) {
			if (data && data.errorCode === 0) {
				cvUtil.toast(cvUtil.cvLocalize("Delete_Aggregator_Success"));
				getAggregators();
			} else {
				cvUtil.errorToast(cvUtil.cvLocalize("Delete_Aggregator_Failure"));
			}

		}, function(xhr, ajaxOptions, thrownError) {
			cvUtil.errorToast(cvUtil.cvLocalize("Delete_Aggregator_Failure"));

		});
	};

	function initCCGrpsDetailPanel(ccGrps) {
		var ccgrpsAvailableList = "#ccGrpsTab .availableList", ccgrpsSelectedList = "#ccGrpsTab .selectedList", cgEle;

		$.each(ccGrps, function(i, cg) {
			cgEle = $(ccgrpsAvailableList + " li[data-id=" + cg[contractUtils.columns.COMMCELL_GROUP_ID] + "]");
			cgEle.appendTo(ccgrpsSelectedList);

		});

		//sort all the lists
		sortList($(ccgrpsAvailableList));
		sortList($(ccgrpsSelectedList));
	}

	function populateCCGrpsList() {
		var deferred = jQuery.Deferred();

		if (entities.ccGrps === undefined) {
			cvUtil.loadPage("getCommCellGroupList.do", '', function(data) {
				entities.ccGrps = data.aaData;
				doPopulate(entities.ccGrps);
				deferred.resolve("true");

			}, function(xhr, ajaxOptions, thrownError) {
				cvUtil.errorToast(cvUtil.cvLocalize("Error_Loading_Licenses"));
				deferred.resolve("true");

			});
		} else {
			doPopulate(entities.ccGrps);
			deferred.resolve("true");
		}
		return deferred.promise();
	}

	function doPopulate(data) {
		var tabId = "ccGrpsTab";
		var availableList = $("#" + tabId + " .availableList");
		var selectedList = $("#" + tabId + " .selectedList");

		availableList.empty();
		selectedList.empty();
		$.each(data, function(i, lic) {
			var sourceLi = $("<li></li>", {
				text : lic.name,
				class : "selectableItem",
				"data-id" : lic.id,
				"data-name" : lic.name,
			});
			availableList.append(sourceLi);
		});
	}

	function addItemsToList(tabid) {
		$("#" + tabid + " .availableList .selectedItem").each(function() {
			$(this).removeClass("selectedItem");
			$(this).appendTo("#" + tabid + " .selectedList");
			$("#" + tabid + " .searchItem").val("");
		});
		//sort the lists
		sortList($("#" + tabid + " .availableList"));
		sortList($("#" + tabid + " .selectedList"));
	}

	function removeItemsFromList(tabid) {
		$("#" + tabid + " .selectedList .selectedItem").each(function() {
			$(this).removeClass("selectedItem");
			$(this).appendTo("#" + tabid + " .availableList");
		});
		//sort the lists
		sortList($("#" + tabid + " .availableList"));
		sortList($("#" + tabid + " .selectedList"));
	}

	function addEvents() {
		$("#manageContractButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/manageContract.jsp");
		});
		$("#partnerRedirectButton").click(function() {
			cvUtil.getTopWindow().location.assign(cvUtil.getContextPath() + "/cloud/contract/partnerManagement.jsp");
		});
		$("#addPartnerButton").click(function() {
			openGroupDetailDialog();
		});
		$("#ccGrpsTab").on("click", ".selectableItem", function() {
			if ($(this).hasClass('selectedItem')) {
				$(this).removeClass('selectedItem');
			} else {
				$(this).addClass('selectedItem');
			}
		});
		$(".addItemButton").click(function() {
			var tabid = $(this).attr('data-tabid');
			$("#messageLabel").text("");
			addItemsToList(tabid);
		});

		$(".removeItemButton").click(function() {
			var tabid = $(this).attr('data-tabid');
			removeItemsFromList(tabid);
		});

		$("#aggregatorName").keypress(function() {
			$("#messageLabel").text("");
		});

		$("#aggregatorDesc").keypress(function() {
			$("#messageLabel").text("");
		});

		$("#aggregatorTableWrapper").on("click",
				'#editItemButton',
				function() {
					var rowIndex = $("#aggregatorTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var rowData = $("#aggregatorTable_table").dataTable().fnGetData()[rowIndex];
					openGroupDetailDialog(rowData);
				});

		$("#aggregatorTableWrapper").on("click",
				'#deleteItemButton',
				function() {
					var rowIndex = $("#aggregatorTable_table").dataTable().fnGetPosition($(this).closest('tr')
							.find('td')[0])[0];
					var rowData = $("#aggregatorTable_table").dataTable().fnGetData()[rowIndex];
					contractUtils.openDeleteConfirmDialog(rowData,
							cvUtil.cvLocalize("Confirm_Aggregator_Deletion"),
							rowData[contractUtils.columns.AGGREGATOR_NAME],
							deleteAggregator);
				});

		$("#aggregatorTableWrapper").on("click", '.tableExportExcel', function() {
			var dataTable = $("#aggregatorTable_table").dataTable();
			contractUtils.exportToExcel(dataTable, contractUtils.global.AGGREGATORS);
		});
	}

})(this);
