 /* =============================================================
 * bootstrap-typeahead.js v2.0.3
 * http://twitter.github.com/bootstrap/javascript.html#typeahead
 * =============================================================
 * Copyright 2012 Twitter, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ============================================================ */


!function($){

  "use strict"; // jshint ;_;


 /*
	 * TYPEAHEAD PUBLIC CLASS DEFINITION =================================
	 */

  var Typeahead = function (element, options) {
    this.$element = $(element);
    this.options = $.extend({}, $.fn.typeahead.defaults, options);
    this.matcher = this.options.matcher || this.matcher;
    this.sorter = this.options.sorter || this.sorter;
    this.highlighter = this.options.highlighter || this.highlighter;
    this.updater = this.options.updater || this.updater;
    this.closeOnSelect = this.options.closeOnSelect;
    this.$menu = $(this.options.menu).appendTo('body');
    this.source = this.options.source;
    this.noAutoExpand = this.options.noAutoExpand;
    this.shown = false;
    this.controlblur = false;
    this.counter = 0;
    this.clickh = true;
    this.hidetimeout={};
    this.listen();
    this.activeComp = false;
    this.reset = this.options.reset || this.reset;
    this.hideOnSelect = this.options.hideOnSelect;
  };

  Typeahead.prototype = {

    constructor: Typeahead

  , select: function () {
	  this.controlblur = true;
	  this.counter=1;
	  var val = this.$menu.find('.active').attr('data-value');
	  var valObj = null;
      if(typeof this.options.source[0] == 'object'){
        val = $.map(this.options.source,function(value,i){
            if(value.label == val) {
				return value.value;
			}
        });
      }

      if (typeof this.source[0] == 'object') {
    	  valObj = $.map(this.source,function(value,i){
              if(value.label == val) {
  				return value;
  			}})
      }

      if(this.options.delimiter !== ""){
    	  var prevVal = this.$element.val();
    	  val = prevVal.slice(0,prevVal.lastIndexOf(this.options.delimiter)+1) + val;
      }

      if (valObj) {
    	  valObj = valObj[0];
      }
      this.$element
        .val(this.updater(val+this.options.delimiter, valObj))
        .change();

      this.$element.focus();
      this.clickh = true;
      if (this.hideOnSelect) {
    	  this.hide();
      }
      return this;
    }

  , updater: function (item) {
      return item;
    }

  , show: function () {
      var pos = $.extend({}, this.$element.offset(), {
        height: this.$element[0].offsetHeight
      });

      /*
		 * if(!(this.$element[0].offsetParent == undefined || this.$element[0].offsetParent == null) &&
		 * this.$element[0].offsetParent.nodeName != "TD"){ pos.left +=
		 * this.$element[0].offsetParent.offsetLeft; }
		 */

      this.$menu.css({
        top: pos.top + pos.height
      , left: pos.left
      });
      $(".typeahead").hide();
      this.$menu.show();
      this.shown = true;
      return this;
    }

  , hide: function () {
	  if(this.counter == 0){
		  this.$menu.hide();
		  this.shown = false;
	  }
	  if(this.controlblur){
		  this.controlblur = false;
	  }else{
		  this.$menu.hide();
		  this.shown = false;
	  }
	  this.counter=0;
      return this;
    }
  , setSource: function(source) {
	  this.source = source;
  }
  ,lookup: function(event) {
      if (this.options.sourceCallback && typeof this.options.sourceCallback == "function") {
    	  this.options.sourceCallback(this, this.query)
      } else {
    	  this.process();
      }
  }
  , process: function (dataSource, isRefresh) {
	  var that = this
      , items;

	  if (dataSource) {
		  this.source = dataSource;
	  }



	  if(this.options.delimiter === ""){
		  this.query = this.$element.val().split(',');
	  }else{
		  this.query = this.$element.val().split(this.options.delimiter);//added by snaskar
	  }
    this.query =  (this.query.length === 0) ? "" : this.query[this.query.length - 1];

    /*
	 * if (!this.query) { return this.shown ? this.hide() : this; }
	 */

      if (!this.query){
    	items= $.grep(this.source, function (item) {
	        return true;
	    });
	  }else{
	      items = $.grep(this.source, function (item) {
	        return that.matcher(item);
	      });
	  }


    /* added by snaskar */
	  var selectedVals;
	  selectedVals = this.$element.val().split(this.options.delimiter);

	  //console.log(selectedVals);
	  selectedVals = selectedVals.slice(0,selectedVals.length-1);
	  //console.log(selectedVals);
	  /* end */

    var lbls = new Array();
    var x=0;
    for(var i=0;i<items.length;i++){
        if(typeof items[i] == 'object'){
      	  /* added by snaskar */

  		  if(selectedVals.indexOf(items[i].label) != -1){
  			  continue;
  		  }/* end */
  		  lbls[x++] = items[i].label;

        }

    }
    /* if(lbls.length != 0){ */
        items = lbls;
    /* } */
    //items = getlabels(items)

    items = this.sorter(items);

    if (!items.length) {
      return this.shown ? this.hide() : this;
    }
    return this.render(items.slice(0, this.options.items)).show();
    }

  , matcher: function (item) {
      if(typeof item == 'object'){
          return ~item.label.toLowerCase().indexOf(this.query.toLowerCase());
      }else{
        return ~item.toLowerCase().indexOf(this.query.toLowerCase());
      }
    }

  , sorter: function (items) {
	  if(items.length == 0){
		  return items;
	  }
      var beginswith = []
        , caseSensitive = []
        , caseInsensitive = []
        , item;

      while (item = items.shift()) {
        if (!item.toLowerCase().indexOf(this.query.toLowerCase())){beginswith.push(item);}else if (~item.indexOf(this.query)){caseSensitive.push(item);} else {caseInsensitive.push(item);}
      }

      return beginswith.concat(caseSensitive, caseInsensitive);
    }

  , highlighter: function (item) {
      var query = this.query.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g, '\\$&');
      return item.replace(new RegExp('(' + query + ')', 'ig'), function ($1, match) {
        return '<strong>' + match + '</strong>';
      });
    }

  , reset : function() {
	  this.activeComp = false;
  	}
  
  , render: function (items) {
      var that = this;
      items = $(items).map(function (i, item) {
        i = $(that.options.item).attr('data-value', item);
        i.find('a').html(that.highlighter(item));
        return i[0];
      });

	  items.first().addClass('active');
      this.$menu.html(items);
      return this;
    }

  , next: function (event) {
      var active = this.$menu.find('.active').removeClass('active')
        , next = active.next();

      if (!next.length) {
        next = $(this.$menu.find('li')[0]);
      }

      next.addClass('active');
      this.$menu.scrollTop($(next).index() * $(next).outerHeight() - (this.$menu.height())+$(next).outerHeight()*2.5);
    }

  , prev: function (event) {
      var active = this.$menu.find('.active').removeClass('active')
        , prev = active.prev();

      if (!prev.length) {
        prev = this.$menu.find('li').last();
      }

      prev.addClass('active');
      this.$menu.scrollTop($(prev).index() * $(prev).outerHeight());
    }

  , listen: function () {
      this.$element
        .on('blur',     $.proxy(this.blur, this))
        .on('keypress', $.proxy(this.keypress, this))
        .on('keyup',    $.proxy(this.keyup, this))
        .on('focus', $.proxy(this.targetfocus, this));

      if ($.browser.webkit || $.browser.msie) {
        this.$element.on('keydown', $.proxy(this.keypress, this));
      }

      this.$menu
        .on('click', $.proxy(this.click, this))
        .on('mouseenter', 'li', $.proxy(this.mouseenter, this));
    }

  , keyup: function (e) {
      switch(e.keyCode) {
        case 40: // down arrow
        case 38: // up arrow
          break;

        case 9: // tab
        case 13: // enter
          if (!this.shown) {
			return

		}          this.clickh = false;
          this.select();
          break;

        case 27: // escape
          if (!this.shown) {
			return

		}          this.clickh = true;
          this.hide();
          break;

        default:
          this.lookup();
      }

      e.stopPropagation();
      e.preventDefault();
  }

  , keypress: function (e) {
      if (!this.shown) {
		return

	}
      switch(e.keyCode) {
        case 9: // tab
        case 13: // enter
        case 27: // escape
          e.preventDefault();
          break;

        case 38: // up arrow
          if (e.type != 'keydown') {
			break;
		}
          e.preventDefault();
          this.prev();
          break;

        case 40: // down arrow
          if (e.type != 'keydown') {
			break;
		}
          e.preventDefault();
          this.next();
          break;
      }

      e.stopPropagation();
    }

  , blur: function (e) {

      //this.hidetimeout = setTimeout(function () { that.hide(); }, 150);
      if(this.hidetimeout){
    	  clearInterval(this.hidetimeout)
      }
	  var that = this;
      this.hidetimeout = setInterval(function () { if(that.clickh){that.hide();clearInterval(that.hidetimeout);} }, 150);
    }

  , click: function (e) {
      //e.stopPropagation();
      //e.preventDefault();
	  this.clickh = false;
      this.select();
    }
  ,targetfocus: function(e){
	  this.controlblur = true;
	  if(! e.view){
		  this.controlblur = false;
		  var elem = document.getElementById(this.$element.attr("id"));
		  if (elem.createTextRange) {
	          var r = elem.createTextRange();
	          r.collapse(true);
	          r.moveEnd("character", elem.value.length);
	          r.moveStart("character", elem.value.length);
	          r.select();
	      }
	      else {
	    	  elem.selectionStart = elem.selectionEnd = elem.value.length;
	      }
	  }
	  
	  if (this.options.noAutoExpand && this.activeComp == false ) {
		  this.activeComp = true;
		  return;
	  }
	  if (this.options.sourceCallback && this.source && this.source.length  > 1 ) {
		  this.process();
	  } else {
		  this.lookup();
	  }

	  //e.stopPropagation();
      //e.preventDefault();

  }

  , mouseenter: function (e) {
      this.$menu.find('.active').removeClass('active');
      $(e.currentTarget).addClass('active');
    }

  };


  /*
	 * TYPEAHEAD PLUGIN DEFINITION ===========================
	 */

  $.fn.typeahead = function (option) {
    return this.each(function () {
      var $this = $(this)
        , data = $this.data('typeahead')
        , options = typeof option == 'object' && option;
      if (!data){$this.data('typeahead', (data = new Typeahead(this, options)));}
      if (typeof option == 'string'){data[option]();}
    });
  };

  $.fn.typeahead.defaults = {
    source: []
  , items: 8
  , menu: '<ul class="typeahead dropdown-menu"></ul>'
  , item: '<li><a href="#"></a></li>'
	  ,delimiter: ';'
  };

  $.fn.typeahead.Constructor = Typeahead;


 /*
	 * TYPEAHEAD DATA-API ==================
	 */

  $(function () {
    $('body').on('focus.typeahead.data-api', '[data-provide="typeahead"]', function (e) {
      var $this = $(this);
      if ($this.data('typeahead')) {
		return

	}      e.preventDefault();
      $this.typeahead($this.data());
    });
  });

}(window.jQuery);