(function () {
    var scatterPlot = function(config, element_id, data) {
        this.init(config, element_id, data);
    };

    scatterPlot.prototype.defaultClickFn = function(elem) {
        // provides a default click function (alert the text)
        alert(elem.data('text'));
    };

    scatterPlot.prototype.showLabel = function(elem){
        // provides a default hover function (show the text)
        var x = elem.getBBox().x,
            y = elem.getBBox().y;

        // CHANGED: Disabled changing stroke color
        // elem.attr('stroke', '#ccc');
        var text = this.r.text(
                elem.getBBox().x,
                elem.getBBox().y - this.config.text_width,
                elem.data('text')).attr(this.config.data_label_style);

        if (x + text.getBBox().width < this.config.size) {
            text.attr('x',
                elem.getBBox().x + (text.getBBox().width / 2) - (this.config.text_width * 1));
        } else {
            text.attr('x',
                elem.getBBox().x - (text.getBBox().width / 2) - (this.config.text_width * 0.5));
        }

        if (y - text.getBBox().height - (this.config.text_width / 2) < 0) {
            text.attr('y',
                elem.getBBox().y + this.config.text_width + (2 * this.config.radius));
        }

        // CHANGED: Forward events on text to corresponding point element,
        // added adaptation to font size
        var self = this;
        elem.data('hover', [
                this.r.rect(
                    text.getBBox().x - (this.config.text_width / 2),
                    text.getBBox().y - (this.config.text_width / 2) + 5,
                    text.getBBox().width + this.config.text_width,
                    text.getBBox().height + this.config.text_width - 10
                ).attr('fill', "#fff")
                // .attr("stroke", "none")
                .attr("cursor", "pointer")
                .click(function() { self.config.clickFn(elem); })
                .hover(
                    function() { self.config.hoverInFn(elem); },
                    function() { self.config.hoverOutFn(elem); }
                ),
                text.toFront()
            ]);
        $(text.node).css("pointer-events", "none");
    };

    scatterPlot.prototype.defaultHoverInFn = scatterPlot.prototype.showLabel;

    scatterPlot.prototype.defaultHoverOutFn = function(elem) {
        // provides a default hover function (hide the text)
        // CHANGED: Disabled changing stroke color
        // elem.attr('stroke', '#333');
        var bits = elem.data('hover');
        elem.data('hover', []);
        for (var i = 0; i < bits.length; i++) {
            bits[i].remove();
        }
    };

    var SIZE = 400;
    scatterPlot.prototype.default_config = {
        size: SIZE,  // plot (i.e. plot area) height/width (square!)
        colours: [  // gradient colours
            ['#CF171F', 0],
            ['#F47721', 37.5],
            ['#FFC80B', 62.5],
            ['#C1D72E', 100]
        ],
        series_colours: ['#000', '#fff'],
        ticks: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],  //ticks.. obv.
        x_label: "",  // x axis label
        y_label: "",  // y axis label
        clickFn: scatterPlot.prototype.defaultClickFn,  // click a point and...
        hoverInFn: scatterPlot.prototype.defaultHoverInFn,  // click a point and...
        hoverOutFn: scatterPlot.prototype.defaultHoverOutFn,  // click a point and...
        // CHANGED: Added config properties
        show_legend: true,
        stroke_colours: ["#555", "#555"],
        stroke_width: 1,
        tick_label_style: { "font-size": 10 },
        axis_label_style: { "font-size": 10 },
        data_label_style: { "font-size": 10 }
    };

    scatterPlot.prototype.getColours = function () {
        // converts our list to an SVG gradient string
        var result = "45-";
        var colours = this.config.colours;
        for (var ci = 0; ci < colours.length; ci++) {
            var datum = colours[ci];
            switch (ci) {
                case 0:
                    result += datum[0];
                    break;
                case colours.length:
                    result += "-" + datum[0];
                    break;
                default:
                    result += "-" + datum[0] + ":" + datum[1];
            }
        }
        return result;
    };

    // CHANGED: Updated to use custom tick visibility list
    scatterPlot.prototype.drawGrid = function() {
        // output the gradient and the lines
        var text_width = this.config.text_width,
            padding = this.config.padding,
            size = this.config.size,
            visible_tick_size = this.config.visible_tick_size,
            visible_ticks = this.config.visible_ticks,
            r = this.r;

        //gradient
        var background = r.rect(padding - text_width, text_width, size, size);
        background.attr('fill', this.getColours());
        var vpad = padding - text_width;
        r.path(  //axes / border
            "M" + (vpad) + "," + (text_width) +  // move to top
            "L" + (vpad) + "," + (size + text_width) +  // line to bottom
            "L" + (vpad + size) + "," + (size + text_width) +  // line to left
            "L" + (vpad + size) + "," + (text_width) +  // line to top
            "L" + (vpad) + "," + (text_width)  // line to right
        );

        // draw the tick lines
        for (var li in visible_ticks) {
            var line = r.path(
                "M" + (vpad) + "," + (text_width + (visible_tick_size * li)) +
                "L" + (vpad + size) + "," + (text_width + (visible_tick_size * li))
            ).attr("stroke-width", 0.5);
            line = r.path(
                "M" + (vpad + (visible_tick_size * li)) + "," + (text_width) +
                "L" + (vpad + (visible_tick_size * li)) + "," + (text_width + size)
            ).attr("stroke-width", 0.5);
        }
    };

    // CHANGED: Updated to use custom tick visibility list
    scatterPlot.prototype.drawLabels = function() {
        // add x, y axis labels and tick labels
        var text_width = this.config.text_width,
            tick_label_style = this.config.tick_label_style,
            padding = this.config.padding,
            size = this.config.size,
            axis_label_style = this.config.axis_label_style,
            visible_tick_size = this.config.tick_size,
            visible_ticks = this.config.ticks,
            r = this.r,
            x_label = this.config.x_label,
            y_label = this.config.y_label,
            xlabels = [],
            ylabels = [];

        // labels
        for (var xi in visible_ticks) {
            if (xi > 0) {
                xlabels.push(
                    r.text(
                        text_width + padding + (xi * visible_tick_size) - (2 * text_width),
                        (2 * text_width) + size,
                        xi).attr(tick_label_style));
            }
        }
        for (var yi in visible_ticks) {
            if (yi > 0) {
                ylabels.push(
                    r.text(
                        padding - (2 * text_width),
                        size - (visible_tick_size * yi) + text_width,
                        yi).attr(tick_label_style));
            }
        }
        // origin
        r.text(padding - (2 * text_width), size + (2 * text_width), 0)
            .attr(tick_label_style);

        // x axis label
        var xl = r.text(
            padding - text_width + (size / 2),
            size + (padding - text_width),
            x_label).attr(axis_label_style);
        // y axis label
        var yl = r.text(
            text_width,
            text_width + (size / 2),
            y_label).attr(axis_label_style);
        yl.rotate(-90);
    };

    scatterPlot.prototype.makeDots = function (data) {
        // update dots on plot (not massively efficient)
        // data sould be a dict of list of lists:
        // data = {
        //  'series 1': [[x1,y1,t1],...],
        //  ...
        //}
        var text_width = this.config.text_width,
            padding = this.config.padding,
            size = this.config.size,
            tick_size = this.config.tick_size,
            ticks = this.config.ticks,
            r = this.r,
            radius = this.config.radius,
            stroke_colours = this.config.stroke_colours,
            stroke_width = this.config.stroke_width,
            clickFn = this.config.clickFn,
            hoverInFn = this.config.hoverInFn,
            hoverOutFn = this.config.hoverOutFn,
            self = this,
            series_colours = this.config.series_colours;

        // remove old ones...
        var len = this.points.length;
        for (var i = 0; i < len; i++) {
            var point = this.points.pop()
            point.remove();
        }
        // add new ones...
        var count = 0;
        for (i in data) {
            var series = data[i];
            for (var j = 0; j < series.length; j++) {
                var datum = series[j];
                var x = datum[0];
                var y = datum[1];
                var t = datum[2];
                var dot = r.circle(
                    padding - text_width + (x * tick_size),
                    size + text_width - (y * tick_size),
                    radius)
                    // CHANGED: Updated to allow changing stroke color and width
                    .attr('stroke', stroke_colours[count])
                    .attr('stroke-width', stroke_width)
                    .attr('fill', series_colours[count])
                    .attr("cursor", "pointer")
                    .click(function() { clickFn(this) })
                    .hover(
                        function() {hoverInFn.call(self, this)},
                        function() {hoverOutFn.call(self, this)});
                for (key in t) {
                    dot.data(key, t[key]);
                }
                this.points.push(dot);
                this.showLabel(dot);
            }
            count++;
        }
    };

    scatterPlot.prototype.showLegend = function(data) {
        var text_width = this.config.text_width,
            padding = this.config.padding,
            size = this.config.size,
            tick_size = this.config.tick_size,
            ticks = this.config.ticks,
            r = this.r,
            radius = this.config.radius,
            series_colours = this.config.series_colours;

        var count = 0;
        var last_rhs = 0;
        for (key in data) {
            var dot = r.circle(
                last_rhs + padding + text_width,
                size + (text_width * 4),
                radius)
                .attr('fill', series_colours[count]);
            var text = r.text(dot.getBBox().x, dot.getBBox().y + radius, key);
            text.attr('x', dot.getBBox().x + text.getBBox().width / 2 + (2 * radius) + text_width);
            count++;
            last_rhs = text.getBBox().x + text.getBBox().width + text_width;
        }
    }

    scatterPlot.prototype.init = function(config, element_id, data) {
        var self = this;
        this.points = [];

        // update config with custom config
        this.config = {};
        // CHANGED: Updated to allow use of config properties which are not in
        // the default config map
        for (var key in this.default_config) {
            this.config[key] = this.default_config[key];
        }
        for (var key in config) {
            this.config[key] = config[key];
        }

        var size = this.config.size;

        // CHANGED: All of these config properties are used if set in the config
        // object, allowing override of previously private properties

        //how far apart are the ticks?
        this.config.tick_size = size / (this.config.ticks.length - 1);
        // CHANGED: Added to allow specification of visible ticks
        if (typeof this.config.visible_ticks === "undefined") {
            this.config.visible_ticks = this.config.ticks;
        }
        this.config.visible_tick_size = size / (this.config.visible_ticks.length - 1);
        // how much LHS/Bottom extra do we need for axes etc?
        if (typeof this.config.padding === "undefined") {
            this.config.padding = size / 10;
        }
        // the dot size
        if (typeof this.config.radius === "undefined") {
            this.config.radius = size / 100;
        }
        //hm, bit rough, but really height/width of chars.
        if (typeof this.config.text_width === "undefined") {
            this.config.text_width = this.config.padding / 4;
        }

        var _getLegendHeight = function() {
            return self.config.text_width * 2;
        }

        var size = this.config.size,
            padding = this.config.padding;

        this.r = Raphael(element_id, size + (2 * padding), size + padding + _getLegendHeight());
        this.drawLabels();
        this.drawGrid();
        if (this.config.show_legend) {
            // CHANGED: Added to enable control of legend display
            this.showLegend(data);
        }
        this.makeDots(data);
    };

    window.scatterPlot = scatterPlot;
})()