(function(proactiveExports) {

	'use restrict';

	var commUniId = cvUtil.getParameter('commUniId', 0);
	var CLIENT_NAMES_URL = '/server/getclndmanames.do';
	var PROACTIVE_URL = '/server/handleProactiveSupport.do';
	var CC_DETAILS_URL = '/server/checkcommcelltroubleshootinginfo.do';
	var tasks_list = [];

	function init() {
		$("#welcomeMsg").text(localMsg.headertext);
		getCommCellDetails();

		$('.submitBtn').on('click', function() {
			if (!isValidInputs()) {
				return;
			}
			updateTasksList();
			submitRequest();
		});

		$(".task-item.custom-cb").on("checkedState", function() {
			enableDisableComponents(this, true);
		}).on("unCheckedState", function() {
			enableDisableComponents(this, false);
		});
		disableAllComponents();
		//$('#client-list').focus();
	}
	function disableAllComponents() {
		enableDisableComponents($('#tools-task'), false);
		enableDisableComponents($('#process-task'), false);
		enableDisableComponents($('#cloud-log-task'), false);
	}

	function enableDisableComponents(el, enable) {
		if ($(el).data("name") === 'tools-task') {
			if (enable) {
				$('#tool-list').removeClass('disabled');
				$('#tool-params').removeClass('disabled');
			} else {
				$('#tool-list').addClass('disabled');
				$('#tool-params').addClass('disabled');
			}
		} else if ($(el).data("name") === 'process-task') {
			if (enable) {
				$('#process-list').removeClass('disabled');
				$('#debug-level').removeClass('disabled');
				$('#max-version').removeClass('disabled');
				$('#max-size').removeClass('disabled');
			} else {
				$('#process-list').addClass('disabled');
				$('#debug-level').addClass('disabled');
				$('#max-version').addClass('disabled');
				$('#max-size').addClass('disabled');
			}
		} else if ($(el).data("name") === 'cloud-log-task') {
			if (enable) {
				$('#log-files-list').removeClass('disabled');
				$('.interval').removeClass('disabled');
			} else {
				$('#log-files-list').addClass('disabled');
				$('.interval').addClass('disabled');
			}
		}
	}

	function errorMsg(error, id, msg) {
		if (error) {
			$(id).addClass('error-task');
			$(id).prop('title', msg);
			cvUtil.errorToast(msg);
		} else {
			$(id).removeClass('error-task');
			$(id).prop('title', '');
		}
	}

	function getCommCellDetails() {
		var data = {};
		data.commUniId = commUniId;
		var url = cvUtil.getContextPath() + CC_DETAILS_URL + '?commUniId=' + commUniId;
		cvUtil.loadPageWithPOST(url, data, function(result) {
			result = cvUtil.parseJSON(result);
			if (result !== undefined) {
				var title = result.commcellname + ' - ' + localMsg.headertext;
				$("#welcomeMsg").text(title);
				$(document).attr("title", title);
				if (result.isVersionSupportForProactive) {
					if (result.isProactiveEnabled) {
						getClientPrpcessNames();
						$(".proactive-container").removeClass('hide');
					} else {
						$(".proactive-container").addClass('hide');
						$("#noDataNotificationPanel").html(result.proactiveNotEnabledMsg);
						$("#noDataNotificationPanel").show();
						return;
					}
				} else {

					if (result.errorCode && result.errorCode != 0) {
						//$(".getloginfo").css("display", "none");
						if (result.errorCode != 999) {
							$("#noDataNotificationPanel").html(checkLocalMsgCodeArrayAndReturnMsg(result.errorCode));
							$("#noDataNotificationPanel").html($("#noDataNotificationPanel").text());
						} else {
							$("#noDataNotificationPanel").html(result.errorString);
						}

					} else {
						$("#noDataNotificationPanel").html(result.proactiveVersionNotSupportedMsg);
						//$("#noDataNotificationPanel").show();
					}
					$(".proactive-container").addClass("hide");
					$("#noDataNotificationPanel").show();
					return;
				}

			}
		}, function(err) {
			cvUtil.errorToast(err.errorString);
		});
	}
	function checkLocalMsgCodeArrayAndReturnMsg(indexToCheck) {
		if (indexToCheck >= localMsgCode.length) {
			return localMsgCode[4];
		} else {
			return localMsgCode[indexToCheck];
		}
	}
	;
	function isValidInputs() {
		var isValid = true;
		var clnames = $.trim($("#client-list").val());

		if (clnames === "" || clnames === null) {
			errorMsg(true, '#client-list', localMsg.select_Client);
			isValid = false;
		} else {
			var patt = new RegExp(",|;");
			if (patt.test(clnames)) {
				errorMsg(true, '#client-list', localMsg.select_Client_error);
				isValid = false;
			} else {
				errorMsg(false, '#client-list');
			}
		}

		var $tasks = $('.task-item.custom-cb');
		var selected = false;
		$.each($tasks, function(i, task) {
			if (isChecked($(task))) {
				selected = true;
			}
		});
		if (!selected) {
			errorMsg(true, '.task-item.custom-cb', localMsg.select_rule);
			isValid = false;
		} else {
			errorMsg(false, '.task-item.custom-cb');

			errorMsg(false, '#tool-list');
			if (isChecked($('#tools-task'))) {
				if ($.trim($('#tool-list').val()) === '') {
					errorMsg(true, '#tool-list', localMsg.select_tool);
					isValid = false;
				}
				errorMsg(false, '#tool-params');
				var toolsParam = $('#tool-params').val();
				if (toolsParam.indexOf('&') >= 0 || toolsParam.indexOf(';') >= 0) {
					errorMsg(true, '#tool-params', localMsg.invalid_tool_param);
					isValid = false;
				}
			}
			errorMsg(false, '#process-list');
			errorMsg(false, "input[type='number']");
			if (isChecked($('#process-task'))) {
				if ($.trim($('#process-list').val()) === '') {
					errorMsg(true, '#process-list', localMsg.select_process);
					isValid = false;
				}
				var dlevel = parseInt($('#debug-level').val());
				var version = parseInt($('#max-version').val());
				var size = parseInt($('#max-size').val());
				if ((isNaN(dlevel) || dlevel < 0) && (isNaN(version) || version < 0) && (isNaN(size) || size < 0)) {
					errorMsg(true, "input[type='number']", localMsg.valid_number);
					isValid = false;
				}
			}
			errorMsg(false, '#log-files-list');
			if (isChecked($('#cloud-log-task'))) {
				if ($.trim($('#log-files-list').val()) === '') {
					errorMsg(true, '#log-files-list', localMsg.log_files);
					isValid = false;
				}
			}
		}
		return isValid;
	}
	function updateTasksList() {
		var clients = $.trim($("#client-list").val());
		var task = {};
		task.clients = clients.replace(/,/g, '');
		if (isChecked($('#tools-task'))) {
			task.tool = {};
			var toolname = $('#tool-list').val();
			task.tool.name = toolname.replace(/,/g, '');
			task.tool.param = $('#tool-params').val();
		}
		if (isChecked($('#process-task'))) {
			task.process = {};
			task.process.name = $('#process-list').val().replace(/.log/g, '');
			var dlevel = parseInt($('#debug-level').val());
			if (!isNaN(dlevel) && dlevel >= 0) {
				task.process.debuglevel = dlevel;
			}
			var version = parseInt($('#max-version').val());
			if (!isNaN(version) && version >= 0) {
				task.process.maxversion = version;
			}
			var size = parseInt($('#max-size').val());
			if (!isNaN(size) && size >= 0) {
				task.process.maxsize = size;
			}
		}
		if (isChecked($('#cloud-log-task'))) {
			task.cloudlogs = {};
			task.cloudlogs.name = $('#log-files-list').val().replace(/,\s+/g, ",");
			if (task.cloudlogs.name.endsWith(",")) {
				task.cloudlogs.name = task.cloudlogs.name.substr(0, task.cloudlogs.name.length - 1);
			}
			task.cloudlogs.interval = Number($("input[name='interval']:checked").val());
		}
		tasks_list.push(task);

		//resetInputs();
	}

	function resetInputs() {
		$("#client-list").val('');

		setChecked($('#tools-task'), false);
		$('#tool-list').val('');
		$('#tool-params').val('');

		setChecked($('#process-task'), false);
		$('#process-list').val('');
		$('#debug-level').val('');
		$('#max-version').val('');
		$('#max-size').val('');

		setChecked($('#cloud-log-task'), false);
		$('#log-files-list').val('');
	}

	function getClientPrpcessNames() {

		var data = {};
		data.commUniId = commUniId;
		data.isProactive = true;
		cvUtil.loadPageWithPOST(cvUtil.getContextPath() + CLIENT_NAMES_URL, data, function(sd) {
			sd = cvUtil.parseJSON(sd);
			if (sd !== null && sd.errorCode !== undefined && sd.errorCode === 0) {
				var entitylist = {};
				entitylist.clients = {};
				entitylist.process = {};
				entitylist.logs = {};
				entitylist.clients.source = sd.CL;
				entitylist.clients.delimiter = "";
				entitylist.process.delimiter = ",";
				entitylist.logs.delimiter = ",";
				$('#client-list').typeahead(entitylist.clients);

				var processList = [], logList = [];
				$.each(sd.BL, function(i, entity) {
					var process = {};
					var label = entity.label.replace('.exe', '.log');
					process.label = label;
					process.value = label;
					processList.push(process);

					var log = {};
					log.label = label;
					log.value = label;
					logList.push(log);
				});
				entitylist.process.source = processList;
				entitylist.logs.source = logList;
				$('#process-list').typeahead(entitylist.process);
				$('#log-files-list').typeahead(entitylist.logs);
				if (sd.tools) {
					var options = '';
					$.each(sd.tools.split(';'), function(i, tool) {
						options += '<option val=' + tool + '>' + tool + '</option>';
					});
					$('#tool-list').append(options);
				}

			} else if (sd !== null && sd.errorCode !== undefined && sd.errorCode !== 999) {
				cvUtil.errorToast(cvUtil.cvLocalize('entities_error', sd.errorCode));
			} else if (sd !== null && sd.errorString !== undefined) {
				cvUtil.errorToast(sd.errorString);
			}
		}, function(errdata) {
			cvUtil.errorToast(errdata.errorString);
			cvUtil.unmask("");
		});
	}

	function submitRequest() {
		var data = {
			'json' : JSON.stringify(tasks_list)
		};
		data.commUniId = commUniId;
		var url = cvUtil.getContextPath() + PROACTIVE_URL + '?commUniId=' + commUniId;
		cvUtil.mask('');
		cvUtil.loadPageWithPOST(url, data, function(resp) {
			cvUtil.unmask('');
			if (resp !== undefined) {
				if (resp.errorCode !== undefined && resp.errorCode === 0) {
					tasks_list = [];
					$('.aftrsbmtmsg').text(cvUtil.cvLocalize('success_msg'));
					$('.aftrsbmtmsg').removeClass('hide');
					resetInputs();
					disableAllComponents();
				} else {
					tasks_list = [];
					$('.aftrsbmtmsg').text(resp.errorString);
					cvUtil.errorToast(resp.errorString);
				}
			}

		}, function(errdata) {
			cvUtil.unmask('');
			tasks_list = [];
			cvUtil.errorToast(cvUtil.cvLocalize('failed_msg', errdata.responseText));
			$('.aftrsbmtmsg').text(cvUtil.cvLocalize('failed_msg', errdata.responseText));
			$('.aftrsbmtmsg').removeClass('hide');
		});
	}

	init();

	proactiveExports.proactiveSupport = {

	};

})(window);
