/*!
* g.Raphael 0.5 - Charting library, based on Raphal
*
* Copyright (c) 2009 Dmitry Baranovskiy (http://g.raphaeljs.com)
* Licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) license.
*/
(function () {
    var mmin = Math.min,
        mmax = Math.max;

    function finger(x, y, width, height, dir, ending, isPath, paper) {
        var path,
            ends = { round: 'round', sharp: 'sharp', soft: 'soft', square: 'square' };

        // dir 0 for horizontal and 1 for vertical
        if ((dir && !height) || (!dir && !width)) {
            return isPath ? "" : paper.path();
        }

        ending = ends[ending] || "square";
        height = Math.round(height);
        width = Math.round(width);
        x = Math.round(x);
        y = Math.round(y);

        switch (ending) {
            case "round":
                if (!dir) {
                    var r = ~~(height / 2);

                    if (width < r) {
                        r = width;
                        path = [
                            "M", x + .5, y + .5 - ~~(height / 2),
                            "l", 0, 0,
                            "a", r, ~~(height / 2), 0, 0, 1, 0, height,
                            "l", 0, 0,
                            "z"
                        ];
                    } else {
                        path = [
                            "M", x + .5, y + .5 - r,
                            "l", width - r, 0,
                            "a", r, r, 0, 1, 1, 0, height,
                            "l", r - width, 0,
                            "z"
                        ];
                    }
                } else {
                    r = ~~(width / 2);

                    if (height < r) {
                        r = height;
                        path = [
                            "M", x - ~~(width / 2), y,
                            "l", 0, 0,
                            "a", ~~(width / 2), r, 0, 0, 1, width, 0,
                            "l", 0, 0,
                            "z"
                        ];
                    } else {
                        path = [
                            "M", x - r, y,
                            "l", 0, r - height,
                            "a", r, r, 0, 1, 1, width, 0,
                            "l", 0, height - r,
                            "z"
                        ];
                    }
                }
                break;
            case "sharp":
                if (!dir) {
                    var half = ~~(height / 2);

                    path = [
                        "M", x, y + half,
                        "l", 0, -height, mmax(width - half, 0), 0, mmin(half, width), half, -mmin(half, width), half + (half * 2 < height),
                        "z"
                    ];
                } else {
                    half = ~~(width / 2);
                    path = [
                        "M", x + half, y,
                        "l", -width, 0, 0, -mmax(height - half, 0), half, -mmin(half, height), half, mmin(half, height), half,
                        "z"
                    ];
                }
                break;
            case "square":
                if (!dir) {
                    path = [
                        "M", x, y + ~~(height / 2),
                        "l", 0, -height, width, 0, 0, height,
                        "z"
                    ];
                } else {
                    path = [
                        "M", x + ~~(width / 2), y,
                        "l", 1 - width, 0, 0, -height, width - 1, 0,
                        "z"
                    ];
                }
                break;
            case "soft":
                if (!dir) {
                    r = mmin(width, Math.round(height / 5));
                    path = [
                        "M", x + .5, y + .5 - ~~(height / 2),
                        "l", width - r, 0,
                        "a", r, r, 0, 0, 1, r, r,
                        "l", 0, height - r * 2,
                        "a", r, r, 0, 0, 1, -r, r,
                        "l", r - width, 0,
                        "z"
                    ];
                } else {
                    r = mmin(Math.round(width / 5), height);
                    path = [
                        "M", x - ~~(width / 2), y,
                        "l", 0, r - height,
                        "a", r, r, 0, 0, 1, r, -r,
                        "l", width - 2 * r, 0,
                        "a", r, r, 0, 0, 1, r, r,
                        "l", 0, height - r,
                        "z"
                    ];
                }
        }

        if (isPath) {
            return path.join(",");
        } else {
            return paper.path(path);
        }
    }

    /*
* Vertical Barchart
*/
  
    function WrappedText(text, height)
    {
            var wText = this;
            wText.text=text;
            wText.height=height;
    }
    
    
    function wrapText(paper,text, maxWidth){
    	var t = paper.text(0, 0).attr('text-anchor', 'start');
    	var tempText = "";
    	var row=0;
    	var rowHeight = t.getBBox().height;
    	var retval = "";
    	
    	for (var i=0; i<text.length; i++) {
    	  t.attr("text", tempText + text[i]);
    	  if (row > 2) {
    		  break;
    	  }
    	  if (t.getBBox().width > maxWidth) {
    	    tempText += "\n" + text[i];
    	    retval += tempText;
    	    // resetting 
    	    t.remove();
    	    t = paper.text(0, 0).attr('text-anchor', 'start');
    	    tempText = "";
    	    
    	    row++;
    	  } else {
    	    tempText += text[i];
    	  }
    	}
    	if(tempText) {
    		retval += tempText;
    	}
    	
    	
    	var wText = new WrappedText(retval, row==0 ?0 : (row *6 ));
    	t.remove();
    	return wText;
    }
    
    function formatPointLabels(paper,labels,maxWidth) {
    	var fLabels = [];
    	for (var i=0; i<labels.length; i++) {
    		fLabels[i] = wrapText(paper,labels[i], maxWidth);
    	}
    	return fLabels;
    }
    
    function VBarchart(paper, x, y, width, height, values, opts, popupLabels, barLabels) {
        opts = opts || {};

        var chartinst = this,
            type = opts.type || "square",
            gutter = parseFloat(opts.gutter || "20%"),
            chart = paper.set(),
            bars = paper.set(),
            covers = paper.set(),
            covers2 = paper.set(),
            total = Math.max.apply(Math, values),
            stacktotal = [],
            multi = 0,
            colors = opts.colors || chartinst.colors,
            len = values.length,
            maxy= opts.maxy || 1,
        	maxx = opts.totalpoints || 1,
        	pointLabels = opts.pointlabels || [],
        	labelxpos = opts.labelxpos || (len *5),
        	labelypos = opts.labelypos || 15,
        	delta=23,
        	barActualWidth = opts.barActualWidth || 40;	
        	serieslabels = opts.serieslabels || {},
        	maxLegendLen = opts.maxLegendLen || 200,
        	barPointLabelWidth = opts.barPointLabelWidth || barActualWidth +20,
        	showLegend = opts.showlegend == undefined ? true : opts.showlegend,
        	topBottomDisplay = opts.topBottomDisplay == undefined ? true :  opts.topBottomDisplay,
        	count = 0;	

        formattedPointLabels = formatPointLabels(paper, pointLabels,barPointLabelWidth);
        formattedLabelHeight = 0;
        for (var i=0; i<formattedPointLabels.length; i++){
        	if (formattedPointLabels[i].height > formattedLabelHeight) {
        		formattedLabelHeight = formattedPointLabels[i].height;
        	}
        }
        
        if (opts.threshold && opts.threshold.label)	 {
        	
        	var text = paper.text(0, 0, opts.threshold.label).attr({fill: '#333', stroke: "none",font:"12px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "400"});
        	var textLen = text.getBBox().width;
        	width -=textLen;
        	text.remove();
        }
      	
        //get width of y axis label
        var text = paper.text(0, 0,Math.round(maxy)).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        var legendLen =text.getBBox().width;
        text.remove();
        x+=legendLen + gutter; 
        var xaxiswidth = (opts.xaxiswidth || width) +4;
        if (labelxpos == -1) {
        	labelxpos = x + xaxiswidth + gutter * 2;
        }
        
        if (Raphael.is(values[0], "array")) {
            total = [];
            multi = len;
            len = 0;

            for (var i = values.length; i--;) {
                bars.push(paper.set());
                total.push(Math.max.apply(Math, values[i]));
                len = Math.max(len, values[i].length);
            }

            if (opts.stacked) {
                for (var i = len; i--;) {
                    var tot = 0;

                    for (var j = values.length; j--;) {
                        tot +=+ values[j][i] || 0;
                    }

                    stacktotal.push(tot);
                }
            }

            for (var i = values.length; i--;) {
                if (values[i].length < len) {
                    for (var j = len; j--;) {
                        values[i].push(0);
                    }
                }
            }

            total = Math.max.apply(Math, opts.stacked ? stacktotal : total);
        }
        
        total = (opts.to) || total;

        var barwidth = barActualWidth || width / (len * (100 + gutter) + gutter) * 100,
            barhgutter = barwidth * gutter / 100,
            barvgutter = opts.vgutter == null ? 20 : opts.vgutter,
            stack = [],
            X = x + barhgutter -5,
            Y = (height - 2 * barvgutter) / total;

        if (!opts.stretch) {
            barhgutter = Math.round(barhgutter);
            barwidth = Math.floor(barwidth);
        }

        !opts.stacked && (barwidth /= multi || 1);

        var axis = paper.set();
        var minx=0,
        	miny=0;

        var axis_opts = {};
        axis_opts.unitstr =  opts.unitstr || "";
        axis_opts.xAxisAddon =X+barwidth - 10;
        axis_opts.isBarChart = true;
        axis_opts.datalength = width -1 * gutter;
        var hideXLabel = opts.axisXHideLabel || false;
        
        
        
        if (opts.title) {
        	var text = paper.text(x, y, opts.title).attr({fill: '#333', stroke: "none",font:"14px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "bold"});
        	var hoverPosX = (width - text.getBBox().width)/2;
         	text.transform('t'+hoverPosX+',0');
        	y+=10;
         	
        }
        if (opts.subTitle) {
        
        	for (var i=0; i< opts.subTitle.length; i++) {
        		y+=5;
        		var subtitle = opts.subTitle[i];
        		var text = paper.text(x, y, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "400"});
            	var hoverPosX = (width - text.getBBox().width)/2;
             	text.transform('t'+hoverPosX+',0');
             	
             	if (subtitle.symbolColor) {
             		paper.rect( text.getBBox().x-20,y -2, 20, 3).attr({fill:subtitle.symbolColor,"stroke-width": 0 });
             	}
        	}
//        	y-=5;
        }
        
        if (opts.xAxisTitle) {
//        	var text = paper.text(x, y + height+ formattedLabelHeight + 15 , opts.xAxisTitle).attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        	var text = paper.text(x, y+80 + height+ formattedLabelHeight + 20 , opts.xAxisTitle).attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        	var hoverPosX = (width - text.getBBox().width)/2;
         	text.transform('t'+hoverPosX+',0').attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});;
        }
        
//        if (opts.grid){
//        	paper.drawGrid(x + gutter, y + gutter, width - 2 * gutter, height - 2 * gutter, 24, opts.axisystep, "#EAEAEA");
//        } 
        
//        if (opts.xAxisSubTitle) {
//        	var ypos =  y + height+30;
//        	paper.rect( x,ypos, width, 60).attr({"stroke-width": .1 });
//        	for (var i=0; i< opts.xAxisSubTitle.length; i++) {
//        		var subtitle = opts.xAxisSubTitle[i];
//        		var indent = subtitle.indent || 0;
//        		if (subtitle.header) {
//        			paper.rect( x,ypos, width, 20).attr({fill:"#E0E0E0","stroke-width": 0 });
//        			ypos+=10;
//        			paper.text(x +5,ypos, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "400"});
//        		}
//        		else {
//        			var text = paper.text(x,ypos, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "400"});
//                	var hoverPosX = (width - text.getBBox().width)/2;
//                 	text.transform('t'+hoverPosX+',0');
//                 	
//                 	if (subtitle.symbolColor) {
//                 		paper.rect( text.getBBox().x-20,ypos -2, 20, 3).attr({fill:subtitle.symbolColor,"stroke-width": 0 });
//                 	}
//        		}
////        		var text = paper.text(x,ypos, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "400"});
////            	var hoverPosX = (width - text.getBBox().width)/2;
////             	text.transform('t'+hoverPosX+',0');
////             	
////             	if (subtitle.symbolColor) {
////             		paper.rect( text.getBBox().x-20,ypos -2, 20, 3).attr({fill:subtitle.symbolColor,"stroke-width": 0 });
////             	}
//            	ypos+=15;
//        	}
////        	y-=5;
//        }
        
        if (opts.xAxisSubTitle) {
        	var ypos =  y + height+30;
        	paper.rect( x,ypos, width, 60).attr({"stroke-width": .1 });
        	if (opts.xAxisSubTitle.header) {
    			paper.rect( x,ypos, width, 20).attr({fill:"#E0E0E0","stroke-width": 0 });
    			ypos+=10;
    			paper.text(x +5,ypos,opts.xAxisSubTitle.header).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "400"});
    		}
        	if (opts.xAxisSubTitle.items) {
        		ypos +=25;
        		var xpos =x +15;
        		var colWidth = (width -15)/opts.xAxisSubTitle.items.length;
        		for (var i=0; i<opts.xAxisSubTitle.items.length; i++) {
            		var subtitle = opts.xAxisSubTitle.items[i];
            		var text = paper.text(xpos,ypos, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "400"});
            		xpos += colWidth;
//                	ypos+=15;
            	}
        	}
        }
        
        
        if (opts.axis) {
            var ax = (opts.axis + "").split(/[,\s]+/);
            +ax[0] && axis.push(chartinst.axis(x-20 + gutter, y + gutter, width - 2 * gutter, minx, maxx, opts.axisxstep || Math.floor((width - 2 * gutter) / 20), 2,null,0, 2, false, axis_opts,paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            +ax[1] && axis.push(chartinst.axis(x-20 + width - gutter, y + height - gutter, height - 2 * gutter, miny, maxy, opts.axisystep || Math.floor((height - 2 * gutter) / 20), 3,  null, 0, 2,false,paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            +ax[2] && axis.push(chartinst.axis(x-20 + gutter, y + height - barvgutter,xaxiswidth, minx, maxx, opts.axisxstep || Math.floor((width - 2 * gutter) / 20), 0, null,null, 1, true,axis_opts,paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            +ax[3] && axis.push(chartinst.axis(x-20 + gutter, y + height + 50 - gutter, height+ 30, miny, maxy, 1 || Math.floor((height - 2 * gutter) / 20), 1,null, 0, 1,false,axis_opts, paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            
	    	var x1 = 60;
	    	var x2 = x + 11;//(width + gutter *2)-2;
	    	var ky = (height - gutter * 2) / maxy;
	    	var y1 = y + height + 50 - gutter;
	    	for(var i=0;i<24;i++){
	    			var realpath = [["M"+x2],[y1,"L",x2],[23,"M"]];
			    	var path =  [["M",0,x2 ], ["L",0,y1]];
			    	var tColor = "#E0E0E0";
			    	var line = paper.path().attr({path: realpath}).attr({"stroke":tColor,"stroke-width": 0.5});
			    	x2 = x2 + 32;
	    	}

/*            for(var i =0;i<24;i++){
            	if(i%2){
            		
            		axis.push(chartinst.axis(x-20 + i*20 + gutter, y + height + 50 - gutter, height+ 30,"", "", 1 || Math.floor((height - 2 * gutter) / 20), 1,null, 0, 1,false,axis_opts, paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            		x += 20;
            	} else {
            		
            		axis.push(chartinst.axis(x-20 + i*20 + gutter, y + height + 50 - gutter, height+ 30, "", "", 1 || Math.floor((height - 2 * gutter) / 20), 1,null, 0, 1,false,axis_opts, paper).attr({fill: "#d6d6d6",stroke: "#d6d6d6"}));
            	}
            }*/
        }
        
        if (showLegend && opts.serieslabels) {
	        	  for (var j = 0; j < (multi || 1); j++) {
	        		  var sqre = paper.rect(labelxpos-30,labelypos + (delta*j)-4, 15, 6).attr({fill:colors[j],"stroke-width": 0 }),
	        		  txt = paper.text(labelxpos, labelypos + (delta*j), serieslabels[j]).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": 400});
	        	  }
        }
        
        if (opts.notes) {
        	 if (opts.serieslabels)
        		 var d = delta * opts.serieslabels.length + 10;
        	 paper.text(labelxpos - 30 , labelypos + d   , opts.notes).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": 400});
        }

        if (opts.yAxisTitle) {        	
        	var yTitleXPos= opts.yAxisTitleXPos || x-legendLen - gutter ;
        	var yText = paper.text(0, y + (height/2),opts.yAxisTitle);
        	var newY =yText.getBBox().width/2;
        	yText.transform("t" + yTitleXPos + "," + newY+ "r-90").attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        }
        
		if (opts.threshold)
	    {
	    	var x1 = x+5;
	    	var x2 = x1 + xaxiswidth -2;//(width + gutter *2)-2;
	    	var ky = (height - gutter * 2) / maxy;
	    	var y1 = y + height - gutter - (opts.threshold.value ) * ky;
	    	
	    	var path =  [["M", x1,y1 ], ["L",x2,y1]];
	    	var tColor = opts.threshold.color || "#FF0000";
	    	var line = paper.path().attr({path: path}).attr({"stroke":tColor,"stroke-width": 1.5});
	    	if (opts.threshold.label)
	    		paper.text(x2 + 8, y1, opts.threshold.label).attr({stroke: "none",font:"12px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": 400});
		    
	    }
    
//		if (topBottomDisplay == true)  {
//			 for (var i = 0; i < len; i++) {
//				 if (opts.grouped) {
//					 var group = parseInt(opts.grouped
//					 
//				 }
//		       drawVBar(i); 	
//		      }
//		} else {
//			for (var i = len-1; i >=0; i--){
//			       drawVBar(i); 	
//			 }
//		}
//      

		var group =1;
		 if (opts.grouped) {
			 group = parseInt(opts.grouped);
		 }
		 
		 
		if (topBottomDisplay == true)  {
			if (group >1) {
				for (var i = 0; i < len; i++) { 
					for (var k=0;k<group;k++,i++) {//in this code the "I" is incremented one extra. i will try decrementeing after  the loop
				       drawVBar(i); 	
					}
					--i;
					X+=10;
				}
			} else {
				 for (var i = 0; i < len; i++) {
			       drawVBar(i); 	
			      }
			}
		} else {
			for (var i = len-1; i >=0; i--){
			       drawVBar(i); 	
			 }
		}
        covers2.toFront();
        X = x + barhgutter;

        if (!opts.stacked) {
            for (var i = 0; i < len; i++) {
                for (var j = 0; j < (multi || 1); j++) {
                    var cover;

                    covers.push(cover = paper.rect(Math.round(X), y + barvgutter, barActualWidth, height - barvgutter).attr(chartinst.shim));
                    cover.bar = multi ? bars[j][i] : bars[i];
                    cover.value = cover.bar.value;
                    X += barwidth;
                }

                X += barhgutter;
            }
        }
        


        function drawVBar(i) {
            stack = [];
            if (i==0)
            	X +=4;
            var myY = (height - 2 * barvgutter) / (opts.maxy || total);
            for (var j = 0; j < (multi || 1); j++) {
            	var tempH = multi ? values[j][i] : values[i];
                var h = Math.round((multi ? values[j][i] : values[i]) *  myY);
                if (tempH >0 && h==0) {
                	h=2;
                }
                	
//                var h = Math.round((multi ? values[j][i] : values[i]) *  Y),
                    var top = y + height - barvgutter -h,
                    bar = finger(Math.round(X + barActualWidth / 2), top + h, barActualWidth, h, true, type, null, paper).attr({ stroke: "none", fill: colors[count%4] });
                    
                    count++;
                if (multi) {
                    bars[j].push(bar);
                } else {
                    bars.push(bar);
                }

                bar.y = top;
                bar.x = Math.round(X + barActualWidth / 2);
                bar.w = barActualWidth;
                bar.h = h;
                bar.value = multi ? values[j][i] : values[i];
                if (popupLabels)
                	bar.popupLabel = popupLabels[j][i];
                
                if ( bar.h !=0 && barLabels) {
                	paper.text(bar.x-gutter, bar.y-10, barLabels[j][i]).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "500"});
                }

                if (!opts.stacked) {
                    X += barActualWidth;
                } else {
                    stack.push(bar);
                }
            }

            if (opts.stacked) {
                var cvr;

                covers2.push(cvr = paper.rect(stack[0].x - stack[0].w / 2, y, barActualWidth, height).attr(chartinst.shim));
                cvr.bars = paper.set();

                var size = 0;

                for (var s = stack.length; s--;) {
                    stack[s].toFront();
                }

                top = y + height - barvgutter - h;
                for (var s = 0, ss = stack.length; s < ss; s++) {
                    var bar = stack[s],
                        cover, 
                        h = bar.value * myY;
                    var tempH;
                    if (bar.value >0)
                    	  h = (size + bar.value) * myY;
                    	
                    tempH=   Math.round(h);
                    if (tempH==0 && h >0) {
                    	h=2;
                    }
                    	
                    var path = finger(bar.x, y + height - barvgutter - !!size * .5, barActualWidth, h, true, type, 1, paper);
                     
                   cvr.bars.push(bar);
                    size && bar.attr({path: path});
                    bar.h = h;
                    bar.y = y + height - barvgutter - !!size * .5 - h;
                    if (bar.value > 0)
                    	covers.push(cover = paper.rect(bar.x - bar.w / 2, bar.y , barActualWidth, bar.value * myY ).attr(chartinst.shim));
                    else {
                    	var valAbs = Math.abs(bar.value);
                    	covers.push(cover = paper.rect(bar.x - bar.w / 2,(y + height - barvgutter)   , barActualWidth,valAbs* myY ).attr(chartinst.shim));
                    }
                    cover.bar = bar;
                    cover.value = bar.value;
                    if ( bar.value  >0)
                    	size += bar.value ;
                }
                X += barwidth;
            }
            X += barhgutter;
        }
        
        
        
        chart.label = function (labels) {
        	labels = labels || [];
        	this.labels = paper.set();
        	
            if (topBottomDisplay== false) {
                    for (var i = len-1, j= 0; i >=0 ; i--, j++) {
                    	this.labels.push(writeVLabel(i,j, labels, bars));
                    	
                    }
            } else {
            	for (var i = len-1,  j= len-1; i >=0; i--, j--) {
            		this.labels.push(writeVLabel(i,j, labels, bars));
            	}
            }
            
        	return this;
        };
        
        
        function writeVLabel(i,j, labels, bars) {
        	var L;
    		  if (formattedPointLabels && formattedPointLabels.length >0) {
	      		  var label = formattedPointLabels[i].text == "" ? "" : paper.raphael.g.labelise(formattedPointLabels[i].text, multi ? values[0][i] : values[i], total);
	                L = paper.text(bars[0][j].x ,  y + formattedPointLabels[i].height + height - barvgutter / 2, label).insertBefore(covers[0]);//covers[i * (multi || 1) + j]);
    		  } else {
    			var label =  paper.raphael.g.labelise(labels[i], multi ? values[0][i] : values[i], total);
    			L = paper.text(bars[0][j].x , y + height - barvgutter / 2, label).insertBefore(covers[0]);//i * (multi || 1) + j]);
    		  }
    		  return L;
       }
        
        
        chart.hover = function (fin, fout) {
            covers2.hide();
            covers.show();
            covers.mouseover(fin).mouseout(fout);
            return this;
        };

        chart.hoverColumn = function (fin, fout) {
            covers.hide();
            covers2.show();
            fout = fout || function () {};
            covers2.mouseover(fin).mouseout(fout);
            return this;
        };

        chart.click = function (f) {
            covers2.hide();
            covers.show();
            covers.click(f);
            return this;
        };

        chart.each = function (f) {
            if (!Raphael.is(f, "function")) {
                return this;
            }
            for (var i = covers.length; i--;) {
                f.call(covers[i]);
            }
            return this;
        };

        chart.eachColumn = function (f) {
            if (!Raphael.is(f, "function")) {
                return this;
            }
            for (var i = covers2.length; i--;) {
                f.call(covers2[i]);
            }
            return this;
        };

        chart.clickColumn = function (f) {
            covers.hide();
            covers2.show();
            covers2.click(f);
            return this;
        };
        
        chart.push(bars, covers, covers2, axis);
        chart.bars = bars;
        chart.covers = covers;
        chart.axis = axis;
        return chart;
    };

    /**
* Horizontal Barchart
*/
    function HBarchart(paper, x, y, width, height, values, opts, popupLabels, barLabels) {
        opts = opts || {};

        var chartinst = this,
            type = opts.type || "square",
            gutter = parseFloat(opts.gutter || "20%"),
            chart = paper.set(),
            bars = paper.set(),
            covers = paper.set(),
            covers2 = paper.set(),
            total = Math.max.apply(Math, values),
            stacktotal = [],
            multi = 0,
            colors = opts.colors || chartinst.colors,
            len = values.length,
            barActualHeight = opts.barActualHeight || 30,
            maxy= opts.maxy || 1,
        	maxx = opts.totalpoints || 1,
        	pointLabels = opts.pointlabels || [],
        	labelxpos = opts.labelxpos || (len *5),
        	labelypos = opts.labelypos || 15,
        	delta=23,
        	serieslabels = opts.serieslabels || {},
        	maxLegendLen = opts.maxLegendLen || 50,
        	pointLabelX = x,
        	showLegend = opts.showlegend == undefined ? true : opts.showlegend,
        	topBottomDisplay = opts.topBottomDisplay == undefined ? true :  opts.topBottomDisplay;		

        var legendLen = 0;
        
        for (var i=0; i<pointLabels.length; i++) {
        	var text = paper.text(0, 0,pointLabels[i]).attr({ "text-anchor": 'start', font: "11px 'Fontin Sans',Fontin-Sans,sans-serif"});
        	if (text.getBBox().width >  legendLen) 
        		legendLen =text.getBBox().width;
        	text.remove();
        }
      	//width=swidth-  legendLen-x-gutter;
      	var xaxiswidth = (opts.xaxiswidth || width) +4;
        x+=legendLen + gutter; 
        if (labelxpos == -1) {
        	labelxpos = x + width + gutter * 2;
        }
        
        if (Raphael.is(values[0], "array")) {
            total = [];
            multi = len;
            len = 0;

            for (var i = values.length; i--;) {
                bars.push(paper.set());
                total.push(Math.max.apply(Math, values[i]));
                len = Math.max(len, values[i].length);
            }

            if (opts.stacked) {
                for (var i = len; i--;) {
                    var tot = 0;
                    for (var j = values.length; j--;) {
                        tot +=+ values[j][i] || 0;
                    }
                    stacktotal.push(tot);
                }
            }

            for (var i = values.length; i--;) {
                if (values[i].length < len) {
                    for (var j = len; j--;) {
                        values[i].push(0);
                    }
                }
            }

            total = Math.max.apply(Math, opts.stacked ? stacktotal : total);
        }
        
        total = (opts.to) || total;
        var barheight = Math.floor(height / (len * (100 + gutter) + gutter) * 100),
            bargutter = Math.floor(barheight * gutter / 100),
            stack = [],
            X = (width - 1 * gutter) / (opts.maxy || total);
        
        var axis = paper.set();
        var minx=0,
        	miny=0;

        var axis_opts = {};
        axis_opts.unitstr =  opts.unitstr || "";
        axis_opts.datalength = width -1 * gutter;
        
        if (opts.title) {
        	var text = paper.text(x, y, opts.title).attr({fill: '#333', stroke: "none",font:"14px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "bold"});
        	var hoverPosX = (width - text.getBBox().width)/2;
         	text.transform('t'+hoverPosX+',0');
        	y+=10;
        }
        if (opts.subTitle) {
        
        	for (var i=0; i< opts.subTitle.length; i++) {
        		y+=5;
        		var subtitle = opts.subTitle[i];
        		var text = paper.text(x, y, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start',"font-weight": "400"});
            	var hoverPosX = (width - text.getBBox().width)/2;
             	text.transform('t'+hoverPosX+',0');
             	
             	if (subtitle.symbolColor) {
             		paper.rect( text.getBBox().x-20,y -2, 20, 3).attr({fill:subtitle.symbolColor,"stroke-width": 0 });
             	}
        	}
        }
        
        var Y = y + (bargutter*2);
        
        
        if (opts.yAxisTitle) {
        	var text = paper.text(x, y + height+ gutter , opts.yAxisTitle);
        	var hoverPosX = (width - text.getBBox().width)/2 ;
        	text.transform('t'+hoverPosX+',0').attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        }
        
        if (opts.xAxisTitle) {
     	   var yTitleXPos= opts.yAxisTitleXPos || x-legendLen -gutter -30;
     	  var yTitleYPos=y + gutter + (height/2);
     	   var xText =paper.text(yTitleXPos,yTitleYPos , opts.xAxisTitle);
     	   var newY =xText.getBBox().width/2;
     	   xText.transform( "t10," + newY+ "r-90").attr({fill: '#333', stroke: "none",font:"13px 'Fontin Sans',Fontin-Sans,sans-serif","text-anchor": 'start'});
        }
        
        
        if (opts.xAxisSubTitle) {
        	var ypos =  y + height+30;
        	paper.rect( x,ypos, width, 60).attr({"stroke-width": .1 });
        	if (opts.xAxisSubTitle.header) {
    			paper.rect( x,ypos, width, 20).attr({fill:"#E0E0E0","stroke-width": 0 });
    			ypos+=10;
    			paper.text(x +5,ypos,opts.xAxisSubTitle.header).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "400"});
    		}
        	if (opts.xAxisSubTitle.items) {
        		ypos +=25;
        		var xpos =x +15;
        		var colWidth = (width -15)/opts.xAxisSubTitle.items.length;
        		for (var i=0; i<opts.xAxisSubTitle.items.length; i++) {
            		var subtitle = opts.xAxisSubTitle.items[i];
            		var text = paper.text(xpos,ypos, subtitle.text).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "400"});
            		xpos += colWidth;
//                	ypos+=15;
            	}
        	}
        }
        if (opts.axis) {
            var ax = (opts.axis + "").split(/[,\s]+/);
            +ax[0] && axis.push(chartinst.axis(x-20 + gutter, y+ gutter, width - 2 * gutter, minx, maxx, opts.axisxstep || Math.floor((width - 2 * gutter) / 20), 2,null,0, 2, false, axis_opts,paper));
            +ax[1] && axis.push(chartinst.axis(x-20 + width - gutter, y + height - gutter, height - 2 * gutter, miny, maxy, opts.axisystep || Math.floor((height - 2 * gutter) / 20), 3,  null, 0, 2,false,paper));
//            +ax[2] && axis.push(chartinst.axis(x-20 + gutter, y + height - gutter,xaxiswidth-1*gutter, minx, maxx, opts.axisxstep || Math.floor((width - 2 * gutter) / 20), 1, null,null, 1, false,axis_opts,paper));
//            +ax[3] && axis.push(chartinst.axis(x-20 + gutter, y + height - gutter, height - 2 * gutter, miny, maxy, opts.axisystep || Math.floor((height - 2 * gutter) / 20), 0,null, 0, 1,true,axis_opts, paper));
            +ax[2] && axis.push(chartinst.axis(x-21 + gutter, y + height,height -  gutter + 5, minx, maxx, opts.axisxstep || Math.floor((width - 2 * gutter) / 20), 1, null,null, 1, true,axis_opts,paper));
          +ax[3] && axis.push(chartinst.axis(x , y + height -5, width-1*gutter, miny, maxy, opts.axisystep || Math.floor((height - 2 * gutter) / 20), 0,null, 0, 1,false,axis_opts, paper));
        }
        
        if (showLegend && opts.serieslabels) {
        	  for (var j = 0; j < (multi || 1); j++) {
        		  var sqre = paper.rect(labelxpos-30,labelypos + (delta*j)-4, 15, 6).attr({fill:colors[j],"stroke-width": 0 }),
        		  txt = paper.text(labelxpos, labelypos + (delta*j), serieslabels[j]).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": 400});
        	  }
        }
        
        if (opts.notes) {
       	 if (opts.serieslabels)
       		 var d = delta * opts.serieslabels.length + 10;
       	 paper.text(labelxpos - 30 , labelypos + d   , opts.notes).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": 400});
       }

    
    	
   
       
         if (topBottomDisplay==true) {
        	 for (var i = 0; i<len; i++) {
            		drawHBar(i);
        	 }
         } else {
        	 for (var i = len-1; i >=0 ; i--) {
            		drawHBar(i);
            }
         }

        covers2.toFront();
        Y = y + bargutter;

        if (!opts.stacked) {
            for (var i = 0; i < len; i++) {
                for (var j = 0; j < (multi || 1); j++) {
                    var cover = paper.rect(x, Y, width, barActualHeight).attr(chartinst.shim);

                    covers.push(cover);
                    cover.bar = multi ? bars[j][i] : bars[i];
                    cover.value = cover.bar.value;
                    Y += barheight;
                }

                Y += bargutter;
            }
        }
        
        function drawHBar(i) {
        	 stack = [];
             for (var j = 0; j < (multi || 1); j++) {
                 var val = multi ? values[j][i]   : values[i] ;
                 var displayVal =val * X;
                 if (Math.round(displayVal) ==0 && val >0) {
                	 displayVal =2;
                 }
                     bar = finger(x, Y + barheight / 2, displayVal, barActualHeight - 1, false, type, null, paper).attr({stroke: "none", fill: colors[multi ? j : i]});

                 if (multi) {
                     bars[j].push(bar);
                 } else {
                     bars.push(bar);
                 }


                 bar.x = x + Math.round(displayVal);
                 bar.y = Y + barheight / 2;
                 bar.w = Math.round(displayVal);
                 bar.h = barActualHeight;
                 bar.value =multi ? values[j][i] : values[i];;

                 if (popupLabels)
                 	bar.popupLabel = popupLabels[j][i];
                 
                 if ( bar.h !=0 && barLabels) {
                 	paper.text(bar.x-gutter, bar.y-10, barLabels[j][i]).attr({fill: '#333', stroke: "none",font:"11px Helevetica,Arial,Sans-serif","text-anchor": 'start',"font-weight": "500"});
                 }
                 
                 if (!opts.stacked) {
                     Y += barheight;
                 } else {
                     stack.push(bar);
                 }
             }

             if (opts.stacked) {
                 var cvr = paper.rect(x, stack[0].y - stack[0].h / 2, width, barActualHeight).attr(chartinst.shim);

                 covers2.push(cvr);
                 cvr.bars = paper.set();

                 var size = 0;

                 for (var s = stack.length; s--;) {
                     stack[s].toFront();
                 }

                 for (var s = 0, ss = stack.length; s < ss; s++) {
                     var bar = stack[s],
                         cover,
                         tempVal = (size + bar.value) * X,
                         val = Math.round(tempVal);
                     	if (val ==0 && tempVal >0) {
                     		val =2;
                     	}
                         var path = finger(x, bar.y, val, barActualHeight - 1, false, type, 1, paper);
//                         path = finger(x, bar.y, bar.value * X, barActualHeight - 1, false, type, 1, paper);
                         
                     

                     cvr.bars.push(bar);
                     size && bar.attr({ path: path });
                     bar.w = val;
                     bar.x = x +val;
                     covers.push(cover = paper.rect(x + size * X, bar.y - bar.h / 2,val, barActualHeight).attr(chartinst.shim));
//                     covers.push(cover = paper.rect(x + size * X, bar.y - bar.h / 2, bar.value * X, barActualHeight).attr(chartinst.shim));
                     cover.bar = bar;
                     size += bar.value;
                 }

                 Y += barheight;
             }

             Y += bargutter;
        }
        
        chart.label = function (labels, isRight) {
            labels = labels || [];
            this.labels = paper.set();
            var l;

            
            if (topBottomDisplay== true) {
                    for (var i = len-1, j= len-1; i >=0 ; i--, j--) {
                    	this.labels.push(writeHLabel(i,j, labels, bars));
                    	
                    }
            } else {
            	for (var i = 0,  j= len-1; i < len; i++, j--) {
            		this.labels.push(writeHLabel(i,j, labels, bars));
            	}
            }
            return this;
        };

        function writeHLabel(i,j, labels, bars) {
        	 var label = labels[i] == "" ? "" :paper.raphael.g.labelise(labels[i], values[i], total);
             X = x - 10;
             A = "end";
             var L = paper.text(X, bars[0][j].y, label).attr({ "text-anchor": A, font: "11px 'Fontin Sans',Fontin-Sans,sans-serif" }).insertBefore(covers[0]);
             return L;
        }
        
        chart.hover = function (fin, fout) {
            covers2.hide();
            covers.show();
            fout = fout || function () {};
            covers.mouseover(fin).mouseout(fout);
            return this;
        };

        chart.hoverColumn = function (fin, fout) {
            covers.hide();
            covers2.show();
            fout = fout || function () {};
            covers2.mouseover(fin).mouseout(fout);
            return this;
        };

        chart.each = function (f) {
            if (!Raphael.is(f, "function")) {
                return this;
            }
            for (var i = covers.length; i--;) {
                f.call(covers[i]);
            }
            return this;
        };

        chart.eachColumn = function (f) {
            if (!Raphael.is(f, "function")) {
                return this;
            }
            for (var i = covers2.length; i--;) {
                f.call(covers2[i]);
            }
            return this;
        };

        chart.click = function (f) {
            covers2.hide();
            covers.show();
            covers.click(f);
            return this;
        };

        chart.clickColumn = function (f) {
            covers.hide();
            covers2.show();
            covers2.click(f);
            return this;
        };

        chart.push(bars, covers, covers2);
        chart.bars = bars;
        chart.covers = covers;
        return chart;
    };
    
    
    
    var mousemoved = function(e) {
    	if ($("#tip").html() =="")
    		return;
    	$("#tip").css("left", e.pageX+20).css("top", e.pageY+15);
    	$("#tip").stop(true, true).fadeIn(500).stop(true, true).fadeOut(3500);
    };
		
	var mousemoveover = function(e){
		if (this.bar.popupLabel == undefined && this.bar.value ==0)
    		return;
   
		$("#tip").css("left", e.pageX+20).css("top", e.pageY+15);
		var val =undefined;
		if (this.bar.value)
			val = Math.abs(this.bar.value);
		var tipText =  this.bar.popupLabel  || val || "0";
	    $("#tip").html(tipText);
	    $("#tip").stop(true, true).fadeIn(500).stop(true, true).fadeOut(3500);
	    over = true;
		e.stopPropagation();
	};

	var mousemoveout = function(){
		$("#tip").stop(true, true).fadeOut();
		over = false;
	};
	
    //inheritance
    var F = function() {};
    F.prototype = Raphael.g;
    HBarchart.prototype = VBarchart.prototype = new F;
    
    Raphael.fn.hbarchart = function(x, y, width, height, values, opts, popupLabels, barLabels) {
        var hbarChart= new HBarchart(this, x, y, width, height, values, opts,popupLabels, barLabels);
        if (opts && opts.showpopup && opts.showpopup == true) {
        	hbarChart.mousemove(mousemoved);
        	hbarChart.hover(mousemoveover, mousemoveout);
        }
        return hbarChart;
    };
    
  

    
    Raphael.fn.barchart = function(x, y, width, height, values, opts, popupLabels, barLabels) {
        var vBarChart= new VBarchart(this, x, y, width, height, values, opts, popupLabels, barLabels);
        
        if (opts && opts.showpopup && opts.showpopup == true) {
        	vBarChart.mousemove(mousemoved);
        	vBarChart.hover(mousemoveover, mousemoveout);
        }
        return vBarChart;
    };
})();